# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit.api.session_collection import SessionCollection


class Session(RedfishJsonValidator):
    """Super class of Session Service

        Populates self.redfish with Session response.
    """

    SCHEMA_NAME = 'Session'

    def __init__(self, session_id):
        """Session constructor

            Populates self.redfish with Session response.

            Args:
                session_id: The session ID of the user logged in
        """

        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#Session.Session"
        self.redfish["@odata.id"] = \
            SessionCollection.BASE_URI + "/" + str(session_id)
        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = str(session_id)
        self.redfish["Name"] = "User Session"

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import Blueprint
from flask import g

from oneview_redfish_toolkit.api.computer_system_collection \
    import ComputerSystemCollection
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit.services.zone_service import ZoneService

computer_system_collection = Blueprint("computer_system_collection", __name__)


@computer_system_collection.route("/redfish/v1/Systems/", methods=["GET"])
def get_computer_system_collection():
    """Get the Redfish Computer System Collection.

        Get method to return ComputerSystemCollection JSON when
        /redfish/v1/Systems is requested.

        Returns:
                JSON: JSON with ComputerSystemCollection.
    """

    server_profile_list = g.oneview_client.server_profiles.get_all()
    server_profile_list = list(filter(lambda i: i.get('serverHardwareUri'),
                                      server_profile_list))

    server_profile_tmpls = \
        g.oneview_client.server_profile_templates.get_all()

    zone_service = ZoneService(g.oneview_client)
    zone_ids = zone_service.get_zone_ids_by_templates(server_profile_tmpls)

    csc = ComputerSystemCollection(server_profile_list,
                                   server_profile_tmpls,
                                   zone_ids)

    return ResponseBuilder.success(csc)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import time

from hpOneView import HPOneViewException
from jsonschema import ValidationError

from hpOneView.resources.servers.server_profiles import ServerProfiles

from oneview_redfish_toolkit.api.errors import NOT_FOUND_ONEVIEW_ERRORS
from oneview_redfish_toolkit.api.util.power_option import OneViewPowerOption
from oneview_redfish_toolkit import config

DELAY_TO_WAIT_IN_SEC = 3


class ComputerSystemService(object):
    """Represents a Service class of ComputerSystem"""

    def __init__(self, oneview_client):
        """ComputerSystemService constructor

            Args:
                oneview_client: client of Oneview SDK
        """
        self.ov_client = oneview_client

    @staticmethod
    def get_storage_controller(server_profile_tmpl):
        """Returns the local storage controller of Server Profile Template

            If not controller is present, the value returned is None

            Args:
                server_profile_tmpl: the Server Profile Template
        """
        for controller in server_profile_tmpl["localStorage"]["controllers"]:
            has_valid_controller_mode = \
                controller["mode"] == "HBA" or controller["mode"] == "Mixed"

            if controller["deviceSlot"] != "Embedded" and \
                    has_valid_controller_mode:
                return controller

        return None

    def power_off_server_hardware(self, sh_uuid_or_uri, on_compose=False):
        """Power off a Oneview Server Hardware based on app configuration

            Args:
                sh_uuid_or_uri: the UUID or URI of Server Hardware to be
                powered off
                on_compose: if True gets configuration for the situation when
                 want to Compose a System; if False gets configuration for the
                 situation when want to Decompose a System

            Returns:
                True|False: if powering off action was applied to Oneview
        """
        if on_compose:
            config_key = 'PowerOffServerOnCompose'
        else:
            config_key = 'PowerOffServerOnDecompose'

        power_off = config.get_composition_settings()[config_key]

        if power_off:
            power_state = \
                OneViewPowerOption.get_power_state_by_reset_type(power_off)

            self.ov_client.server_hardware.update_power_state(power_state,
                                                              sh_uuid_or_uri)

            return True

        return False

    def validate_computer_system_resource_block_to_composition(self,
                                                               system_block):
        """Validates a Computer System Resource Block to be used in a composition

            Args:
                system_block: the Server Hardware that represents a Computer
                System Resource Block
        """
        if system_block["serverProfileUri"]:
            sp_uuid = system_block["serverProfileUri"].split("/")[-1]
            raise ValidationError(
                "Computer System Resource Block already belongs to a "
                "Composed System with ID {}".format(sp_uuid))

    def create_composed_system(self, server_profile):
        """Creates a Composed System to Redfish (Creating a Server Profile to OV)

            Args:
                server_profile: the Server Profile to be created

            Returns:
                (task, resource_uri): Tuple with task dict from Oneview and
                    resource_uri of server profile created.
                    The resource_uri can be None if the task has errors before
                    the server profile be created.
        """

        task, _ = self.ov_client.connection.post(
            ServerProfiles.URI,
            server_profile)
        resource_uri = task["associatedResource"]["resourceUri"]

        while not task.get("taskErrors") and not resource_uri:
            time.sleep(DELAY_TO_WAIT_IN_SEC)
            task = self.ov_client.tasks.get_by_uri(task["uri"]).data
            resource_uri = task["associatedResource"]["resourceUri"]

        return task, resource_uri

    def get_server_profile_template_from_sp(self, sp_uri):
        """Gets Sever Profile Template uuid from Server Profile uri"""
        all_sp_labels = self.ov_client.labels.get_by_resource(sp_uri)
        server_profile_template_uuid = ""

        for label in all_sp_labels["labels"]:
            try:
                spt_uuid = label["name"].replace(" ", "-")
                is_valid_spt = \
                    self.ov_client.server_profile_templates.get_by_id(
                        spt_uuid).data
                if is_valid_spt:
                    server_profile_template_uuid = spt_uuid
                    break
            except HPOneViewException as e:
                if e.oneview_response["errorCode"] in NOT_FOUND_ONEVIEW_ERRORS:
                    pass
                else:
                    raise

        return server_profile_template_uuid



#!./redfish-venv/bin/python
# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections

import oneview_redfish_toolkit
from oneview_redfish_toolkit.api.event_service import EventService
from oneview_redfish_toolkit.api.redfish_json_validator import \
    RedfishJsonValidator
from oneview_redfish_toolkit.api.session_service import SessionService
from oneview_redfish_toolkit import config


class ServiceRoot(RedfishJsonValidator):
    """Creates a hardcoded ServiceRoot redfish dict

        Populates self.redfish with a hardcoded ServiceRoot values for
        a BladeSystem

    """

    SCHEMA_NAME = 'ServiceRoot'

    def __init__(self, oneview_uuid):
        """Constructor

            Populates self.redfish with a hardcoded ServiceRoot values for
            a BladeSystem. Validates the self.redfish content against the
            ServiceRoot schema

            Parameters:
                oneview_uuid: string containing OneView's UUID
        """

        super().__init__(self.SCHEMA_NAME)
        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = "RootService"
        self.redfish["Name"] = "Root Service"
        self.redfish["RedfishVersion"] = oneview_redfish_toolkit.version()
        self.redfish["UUID"] = oneview_uuid
        self.redfish["Systems"] = collections.OrderedDict()
        self.redfish["Systems"]["@odata.id"] = "/redfish/v1/Systems"
        self.redfish["Chassis"] = collections.OrderedDict()
        self.redfish["Chassis"]["@odata.id"] = "/redfish/v1/Chassis"
        self.redfish["Managers"] = collections.OrderedDict()
        self.redfish["Managers"]["@odata.id"] = "/redfish/v1/Managers"
        self.redfish["CompositionService"] = collections.OrderedDict()
        self.redfish["CompositionService"]["@odata.id"] = \
            "/redfish/v1/CompositionService"

        self.add_event_service_api()
        self.add_session_service_endpoints()

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#ServiceRoot.ServiceRoot"
        self.redfish["@odata.id"] = "/redfish/v1/"
        self.redfish["@Redfish.Copyright"] = oneview_redfish_toolkit.\
            get_copyright()

        self._validate()

    def add_event_service_api(self):
        self.redfish["EventService"] = {"@odata.id": EventService.BASE_URI}

    def add_session_service_endpoints(self):
        self.redfish["SessionService"] = {"@odata.id": SessionService.BASE_URI}
        self.redfish["Links"] = {"Sessions": {}}
        if config.auth_mode_is_session():
            self.redfish['Links']['Sessions']['@odata.id'] = \
                SessionService.BASE_URI + "/Sessions"



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import Blueprint
from flask import g

from oneview_redfish_toolkit.api.resource_block_collection \
    import ResourceBlockCollection
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder

resource_block_collection = Blueprint("resource_block_collection", __name__)


@resource_block_collection.route(
    "/redfish/v1/CompositionService/ResourceBlocks/", methods=["GET"])
def get_resource_block_collection():
    """Get the Redfish ResourceBlock Collection.

        Return ResourceBlockCollection redfish JSON.
        Logs exception of any error and return
        Internal Server Error or Not Found.

        Returns:
            JSON: Redfish json with ResourceBlockCollection.
    """

    # Gets all server hardware
    server_hardware_list = g.oneview_client.server_hardware.get_all()
    server_profile_template_list = g.oneview_client.\
        server_profile_templates.get_all()
    drives_list = g.oneview_client.index_resources \
        .get_all(category="drives", count=10000)
    volume_list = g.oneview_client.volumes.get_all()
    filter_volume_list = [volume for volume in volume_list
                          if volume["isShareable"]]

    # Emptying volume list to suppress external storage changes for
    # current release.
    # In future, remove this line to enable external storage support
    filter_volume_list = []

    # Build ResourceBlockCollection object and validates it
    cc = ResourceBlockCollection(server_hardware_list,
                                 server_profile_template_list,
                                 drives_list, filter_volume_list)

    return ResponseBuilder.success(cc)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Modules own libs
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit import multiple_oneview


def first_parameter_resource(resource, function, *args, **kwargs):
    resource_id = args[0]
    resp = multiple_oneview.query_ov_client_by_resource(
        resource_id, resource, function, *args, **kwargs)
    category_resource.set_map_category_resources_entry(resource_id,
                                                       resource,
                                                       function)
    return resp


def multiple_parameter_resource(resource, function, *args, **kwargs):
    uri_or_id = args[0]
    if 'rest' in uri_or_id:
        get_function = 'get_by_uri'
    else:
        get_function = 'get_by_id'
    temp_args = list(args)
    temp_args.remove(uri_or_id)
    new_args = tuple(temp_args)
    resource_object = multiple_oneview.query_ov_client_by_resource(
        uri_or_id, resource, get_function, uri_or_id)
    resp = multiple_oneview.execute_query_function(
        resource_object, function, *new_args, **kwargs)
    return resp


def filter_uuid_parameter_resource(resource, function, *args, **kwargs):
    if 'filter' not in kwargs:
        return all_oneviews_resource(resource, function, *args, **kwargs)

    resource_id = _get_resource_id_by_filter(kwargs["filter"], None)

    return multiple_oneview.query_ov_client_by_resource(resource_id, resource,
                                                        function, *args,
                                                        **kwargs)


def all_oneviews_resource(resource, function, *args, **kwargs):
    all_results = multiple_oneview.search_resource_multiple_ov(resource,
                                                               function,
                                                               None,
                                                               None,
                                                               *args,
                                                               **kwargs)

    return all_results


def spt_get_all_with_filter(resource, function, *args, **kwargs):
    if 'filter' not in kwargs:
        return all_oneviews_resource(resource, function, *args, **kwargs)

    resource_id = \
        _get_resource_id_by_filter(kwargs["filter"], "enclosureGroupUri")

    return \
        multiple_oneview.query_ov_client_by_resource(resource_id,
                                                     resource,
                                                     function,
                                                     *args, **kwargs)


def drive_enclosures_get_all_with_filter(resource, function, *args, **kwargs):
    if 'filter' not in kwargs:
        return all_oneviews_resource(resource, function, *args, **kwargs)

    resource_id = \
        _get_resource_id_by_filter(kwargs["filter"], "locationUri")

    return \
        multiple_oneview.query_ov_client_by_resource(resource_id,
                                                     resource,
                                                     function,
                                                     *args, **kwargs)


def create_server_profile(resource, function, *args, **kwargs):
    # In this case, args are resulted by the connection.post call;
    # The second index of args represents the resource object;
    # The first index represents the URI to access;
    sp = args[1]
    server_hardware_uri = sp['serverHardwareUri']
    return _run_action(server_hardware_uri, 'server_hardware', 'get_by_uri', resource, function, *args, **kwargs)


def delete_server_profile(resource, function, *args, **kwargs):
    sp_uuid = args[0]
    return _run_action(sp_uuid, 'server_profiles', 'get_by_id', resource,
                       function, *args, **kwargs)


def update_power_state_server_hardware(resource, function, *args, **kwargs):
    sh_uuid = args[1]
    if 'rest' in sh_uuid:
        get_function = 'get_by_uri'
    else:
        get_function = 'get_by_id'

    resource_object = multiple_oneview.query_ov_client_by_resource(
        sh_uuid, resource, get_function, sh_uuid)
    resp = multiple_oneview.execute_query_function(
        resource_object, function, args[0], **kwargs)
    return resp


def create_labels(resource, function, *args, **kwargs):
    resource_id = args[0]["resourceUri"]

    return \
        _run_action(resource_id, 'server_profiles', 'get_by_uri', resource,
                    function, *args, **kwargs)


def _run_action(resource_id, resource_get, function_get, resource,
                function, *args, **kwargs):

    # Check if the resource_id is already mapped
    if not multiple_oneview.get_ov_ip_by_resource(resource_id):
        # Mapping OneView for the resource_id
        multiple_oneview.query_ov_client_by_resource(resource_id,
                                                     resource_get,
                                                     function_get,
                                                     resource_id, **{})

    # Running action on OneView already mapped for resource_id
    return multiple_oneview.query_ov_client_by_resource(resource_id,
                                                        resource, function,
                                                        *args, **kwargs)


def _get_resource_id_by_filter(filter_parameter, resource_property):
    # Check if filter is composed by a list. If it is true
    # sets the resource_id based on a resource_uri;
    # Otherwise just splits the filter string and sets its value
    # into resource_id
    if isinstance(filter_parameter, list):
        for filters in filter_parameter:
            filter_data = filters.split('=')
            if filter_data[0] == resource_property:
                resource_id = filter_data[1]
    else:
        filter_data = filter_parameter.split('=')
        resource_id = filter_data[1]

    return resource_id



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit import config


class SessionService(RedfishJsonValidator):
    """Creates a Session Service dict

        Populates self.redfish with SessionService values.
    """

    SCHEMA_NAME = 'SessionService'
    BASE_URI = '/redfish/v1/SessionService'

    def __init__(self):
        """Constructor

            Populates self.redfish with SessionService response.
        """

        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["@odata.context"] = "/redfish/v1/$metadata" \
            "#SessionService.SessionService"
        self.redfish["@odata.id"] = self.BASE_URI

        self.redfish["Id"] = "SessionService"
        self.redfish["Name"] = "Session Service"
        self.redfish["Description"] = "Session service"

        # Enable Session Service only when authentication mode is session
        if config.auth_mode_is_session():
            self.redfish["ServiceEnabled"] = True
            self.redfish["Sessions"] = {
                "@odata.id": self.BASE_URI + "/Sessions"
            }
        else:
            self.redfish["ServiceEnabled"] = False

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from copy import deepcopy
from flask import abort
from flask import Blueprint
from flask import request
from flask_api import status

from oneview_redfish_toolkit.api.event import Event
from oneview_redfish_toolkit.api.event_service import EventService
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import util

event_service = Blueprint("event_service", __name__)

ONEVIEW_TEST_ALERT = {
    "timestamp": "2018-02-12T20:12:03.231Z",
    "resource": {
        "category": "alerts",
        "associatedResource": {
            "resourceName": "0000A66101, bay 3",
            "resourceUri": "/rest/server-hardware/"
                           "30373737-3237-4D32-3230-313530314752"
        }
    }
}

ONEVIEW_TEST_TASK = {
    "timestamp": "2018-02-12T20:12:03.231Z",
    "resourceUri": "/rest/server-hardware/"
                   "30373737-3237-4D32-3230-313530314752",
    "changeType": None,
    "resource": {
        "category": "server-hardware",
        "name": "0000A66101, bay 3"
    }
}

REDFISH_TO_ONEVIEW_EVENTS = {
    "ResourceAdded": "Created",
    "ResourceUpdated": "Updated",
    "ResourceRemoved": "Deleted"
}


@event_service.route("/redfish/v1/EventService/", methods=["GET"])
def get_event_service():
    """Get the Redfish Event Service.

        Get method to return EventService JSON when
        /redfish/v1/EventService is requested.

        Returns:
            JSON: JSON with EventService.

    """
    evs = EventService(util.get_delivery_retry_attempts(),
                       util.get_delivery_retry_interval())

    return ResponseBuilder.success(evs)


@event_service.route(
    "/redfish/v1/EventService/Actions/EventService.SubmitTestEvent/",
    methods=["POST"])
def execute_test_event_action():
    """Executes the SubmitTestEvent Action

        Return a JSON containing the EventType received.
        Logs exception of any error and return abort.

        Returns:
            JSON: JSON containing the EventType.

        Exceptions:
            Exception: Missing EventType property.
            Return Bad Request status(400)
    """

    if not config.auth_mode_is_conf():
        abort(status.HTTP_404_NOT_FOUND,
              "EventService is not enabled.")

    event_type = None
    try:
        event_type = request.get_json()['EventType']
    except Exception:
        abort(status.HTTP_400_BAD_REQUEST,
              'Invalid JSON data. Missing EventType property.')

    if event_type not in util.get_subscriptions_by_type().keys():
        abort(status.HTTP_400_BAD_REQUEST,
              'Invalid EventType value: %s' % event_type)

    # Creates a sample OneView SCMB message according to
    # the value of 'event_type'
    if event_type == "Alert":
        message = deepcopy(ONEVIEW_TEST_ALERT)
    else:
        message = deepcopy(ONEVIEW_TEST_TASK)
        message['changeType'] = REDFISH_TO_ONEVIEW_EVENTS[event_type]

    event = Event(message)

    util.dispatch_event(event)

    return ResponseBuilder.response(event, status.HTTP_202_ACCEPTED)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# 3rd party libs
from flask import Blueprint
from flask import g

# own libs
from oneview_redfish_toolkit.api.chassis_collection import ChassisCollection
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder


chassis_collection = Blueprint("chassis_collection", __name__)


@chassis_collection.route("/redfish/v1/Chassis/", methods=["GET"])
def get_chassis_collection():
    """Get the Redfish Chassis Collection.

        Return ChassisCollection redfish JSON.
        Logs exception of any error and return
        Internal Server Error or Not Found.

        Returns:
            JSON: Redfish json with ChassisCollection.
            When Server hardware, enclosures or racks is not found
            calls abort(404).
    """

    # Gets all enclosures
    enclosures = g.oneview_client.enclosures.get_all()

    # Gets all racks
    racks = g.oneview_client.racks.get_all()

    # Gets all server hardware
    server_hardware_list = g.oneview_client.server_hardware.get_all()

    # Build Chassis Collection object and validates it
    cc = ChassisCollection(server_hardware_list, enclosures,
                           racks)

    return ResponseBuilder.success(cc)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
import oneview_redfish_toolkit.api.status_mapping as status_mapping


class Chassis(RedfishJsonValidator):
    """Super class of Chassis resources

         Populates self.redfish with common value between Enclosure,
         Rack and Blade.
    """

    SCHEMA_NAME = 'Chassis'

    def __init__(self, oneview_resource):
        """Chassis constructor

        Populates self.redfish with common value between Enclosure,
         Rack and Blade.

        Args:
            oneview_resource: An object some oneview_resource (ServerHardware,
            Enclosure or Rack) to create the Redfish JSON.
        """

        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = oneview_resource["uuid"]
        self.redfish["Name"] = oneview_resource["name"]
        self.redfish["Manufacturer"] = "HPE"
        self.redfish["SerialNumber"] = oneview_resource["serialNumber"]
        self.redfish["PartNumber"] = oneview_resource["partNumber"]
        self.redfish["Status"] = collections.OrderedDict()
        state, health = status_mapping.\
            get_redfish_server_hardware_status_struct(oneview_resource)
        self.redfish["Status"]["State"] = state
        self.redfish["Status"]["Health"] = health
        self.redfish["Links"] = collections.OrderedDict()
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#Chassis.Chassis"
        self.redfish["@odata.id"] = \
            "/redfish/v1/Chassis/" + oneview_resource['uuid']
        self.redfish["Thermal"] = collections.OrderedDict()
        self.redfish["Thermal"]["@odata.id"] = \
            "/redfish/v1/Chassis/" + oneview_resource['uuid'] + "/Thermal"

    def _map_indicator_led(self, uid_state):
        """Maps Oneview's uid state to Redfish's indicator led.

            Maps the known OneView uid state to Redfish indicator led.
            If a unknown uid state shows up it will be mapped to Unknown.

            Args:
                uid_state: Uid state of Oneview.

            Returns:
                string: Redfish indicator led.
        """

        redfish_oneview_indicator_led_map = dict()
        redfish_oneview_indicator_led_map["On"] = "Lit"
        redfish_oneview_indicator_led_map["Off"] = "Off"
        redfish_oneview_indicator_led_map["Blink"] = "Blinking"

        try:
            return redfish_oneview_indicator_led_map[uid_state]
        except Exception:
            return None



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import g

# own libs
from flask_api import status

from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.drive import Drive
from oneview_redfish_toolkit.api.storage import Storage
from oneview_redfish_toolkit.api.volume import Volume
from oneview_redfish_toolkit.api.volume_collection import VolumeCollection
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder

storage = Blueprint("storage", __name__)


@storage.route(ComputerSystem.BASE_URI + "/<uuid>/Storage/1", methods=["GET"])
def get_storage(uuid):
    """Get the Redfish Storage for a given UUID.

        Return Storage Redfish JSON for a given server profile UUID.
        Logs exception of any error and return abort(500)
        Internal Server Error.

        Returns:
            JSON: Redfish json with Storage
            When server profile or server hardware type is not found
            calls abort(404)

        Exceptions:
            Logs the exception and call abort(500)

    """
    server_profile = g.oneview_client.server_profiles.get_by_id(uuid).data
    sht_uri = server_profile['serverHardwareTypeUri']
    server_hardware_type = \
        g.oneview_client.server_hardware_types.get_by_uri(sht_uri).data
    sas_logical_jbods = _find_sas_logical_jbods_by(server_profile)
    external_storage_volumes = [volume for volume in server_profile[
        "sanStorage"]["volumeAttachments"]]

    st = Storage.build_for_composed_system(server_profile,
                                           server_hardware_type,
                                           sas_logical_jbods,
                                           external_storage_volumes)

    return ResponseBuilder.success(st)


@storage.route(ComputerSystem.BASE_URI +
               "/<profile_id>/Storage/1/Drives/<drive_id>",
               methods=["GET"])
def get_drive(profile_id, drive_id):
    """Get the Redfish Storage for a given UUID.

        Return Drive Redfish JSON for a given server profile UUID and Drive ID.
        Logs exception of any error and return abort(500)
        Internal Server Error.

        Returns:
            JSON: Redfish json with Storage
            When server profile is not found calls abort(404)

        Exceptions:
            Logs the exception and call abort(500)

    """

    drive_id_int = None
    logical_jbod = None
    try:
        drive_id_int = int(drive_id)
    except ValueError:
        abort(status.HTTP_400_BAD_REQUEST, "Drive id should be a integer")

    server_profile = g.oneview_client.server_profiles.get_by_id(
        profile_id).data
    sas_logical_jbods = _find_sas_logical_jbods_by(server_profile)

    logical_jbod = _get_logical_jbod(drive_id_int, logical_jbod,
                                     sas_logical_jbods)

    if logical_jbod is None:
        abort(status.HTTP_404_NOT_FOUND, "Drive {} not found"
              .format(drive_id))

    drive_details = Drive.build_for_computer_system(drive_id_int,
                                                    server_profile,
                                                    logical_jbod)

    return ResponseBuilder.success(drive_details)


@storage.route(ComputerSystem.BASE_URI +
               "/<uuid>/Storage/1/Volumes/",
               methods=["GET"])
def get_volumeCollection(uuid):
    """Get the Redfish Volume Collection for a given UUID.

        Return Volume Collection Redfish JSON for a given server profile UUID.

        Returns:
            JSON: Redfish json with Volume Collection
            When Volume Collection is not found calls abort(404)

    """

    server_profile = g.oneview_client.server_profiles.get_by_id(uuid).data

    if len(server_profile["localStorage"]["sasLogicalJBODs"]) == 0 and \
            len(server_profile["sanStorage"]["volumeAttachments"]) == 0:
        abort(status.HTTP_404_NOT_FOUND, "Volumes not found")

    volume_details = VolumeCollection(server_profile)

    return ResponseBuilder.success(volume_details)


@storage.route(ComputerSystem.BASE_URI +
               "/<uuid>/Storage/1/Volumes/<volume_id>",
               methods=["GET"])
def get_volume(uuid, volume_id):
    """Get the Redfish Volume for a given UUID and Volume ID.

        Return Volume Redfish JSON for a given server profile UUID.

        Returns:
            JSON: Redfish json with Volume
            When Volume is not found calls abort(404)

    """
    is_volume_id_integer = _is_volume_id_number(volume_id)
    # volume id can be number in case of sas logical jbod
    # or uuid in case of external storage volume
    if is_volume_id_integer:
        volume_details = Volume.build_volume_details(uuid, volume_id)
    else:
        server_profile = g.oneview_client.server_profiles.get_by_id(uuid).data
        sp_volume = [volume for volume in server_profile[
            "sanStorage"]["volumeAttachments"]
            if volume["volumeUri"].split("/")[-1] == volume_id]
        if sp_volume:
            volume = g.oneview_client.volumes.get_by_id(volume_id).data
            volume_details = Volume.build_external_storage_volume_details(
                uuid, volume, volume_id)
        else:
            abort(status.HTTP_404_NOT_FOUND,
                  "Volume {} not found for Storage 1 of System {}"
                  .format(volume_id, uuid))

    return ResponseBuilder.success(volume_details)


def _get_logical_jbod(drive_id_int, logical_jbod, sas_logical_jbods):
    logical_jbods_sorted = sorted(sas_logical_jbods, key=lambda i: i["uri"])

    count_drives = 0
    for log_jbod in logical_jbods_sorted:
        next_count = count_drives + int(log_jbod["numPhysicalDrives"])

        if drive_id_int in range(count_drives + 1, next_count + 1):
            logical_jbod = log_jbod
            break

        count_drives = next_count

    return logical_jbod


def _find_sas_logical_jbods_by(server_profile):
    sas_logical_jbods = []
    for logical_jbod in server_profile["localStorage"]["sasLogicalJBODs"]:
        if logical_jbod["sasLogicalJBODUri"]:
            item = g.oneview_client.sas_logical_jbods\
                .get(logical_jbod["sasLogicalJBODUri"])
            sas_logical_jbods.append(item)

    return sas_logical_jbods


def _is_volume_id_number(volume_id):
    try:
        int(volume_id)
        return True
    except ValueError:
        return False



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import logging

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import g
from flask import Response
from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Own libs
from oneview_redfish_toolkit.api.network_adapter import NetworkAdapter


network_adapter = Blueprint("network_adapter", __name__)


@network_adapter.route(
    "/redfish/v1/Chassis/<uuid>/NetworkAdapters/<device_id>",
    methods=["GET"])
def get_network_adapter(uuid, device_id):
    """Get the Redfish NetworkInterface for a given UUID and device_id.

        Return NetworkInterface Redfish JSON for a given hardware UUID
        and device_id.

        Parameters:
            uuid: the UUID of the server_hardware
            device_id: The id of the network device

        Returns:
            JSON: Redfish json with NetworkInterface

        Exceptions:
            When hardware is not found calls abort(404)
            When other errors occur calls abort(500)

    """
    try:
        device_id_validation = int(device_id)

        server_hardware = g.oneview_client.server_hardware.get_by_id(uuid).data

        if (device_id_validation - 1) < 0 or (device_id_validation - 1) >= \
            len(server_hardware["portMap"]["deviceSlots"]):
            abort(status.HTTP_404_NOT_FOUND,
                  "Network adapter id {} not found.".format(device_id))

        na = NetworkAdapter(device_id, server_hardware)

        json_str = na.serialize()

        return Response(
            response=json_str,
            status=status.HTTP_200_OK,
            mimetype="application/json")
    except ValueError:
        # Failed to convert device_id to int
        logging.exception(
            "Failed to convert device id {} to integer.".format(device_id))
        abort(status.HTTP_404_NOT_FOUND, "Network adapter not found")
    except HPOneViewException as e:
        # In case of Oneview error log exception and abort
        logging.exception(e)
        if e.oneview_response['errorCode'] == "RESOURCE_NOT_FOUND":
            abort(status.HTTP_404_NOT_FOUND, "Server hardware not found")
        else:
            # Any other Oneview Exception
            abort(status.HTTP_500_INTERNAL_SERVER_ERROR)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
import logging

from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.resource_block import ResourceBlock
from oneview_redfish_toolkit.api import status_mapping
from oneview_redfish_toolkit.api.zone_collection import ZoneCollection
from oneview_redfish_toolkit.services.zone_service import ZoneService


class StorageResourceBlock(ResourceBlock):
    """Creates a ResourceBlock Redfish dict for Storage Drive

        Populates self.redfish with Drive data retrieved from OneView.
    """

    def __init__(self, storage_resource, drive_index_trees, zone_ids,
                 server_profiles):
        """StorageResourceBlock constructor

            Populates self.redfish with the contents of drive

            Args:
                storage_resource: OneView Drive or storage volume dict
                drive_index_trees: Drives index trees dict
                zone_ids: List of Zone Ids that this Resource Block belongs to
                server_profiles: List of Server profiles
        """
        uuid = storage_resource["uri"].split("/")[-1]
        super().__init__(uuid, storage_resource)
        self.redfish["ResourceBlockType"] = ["Storage"]
        self.redfish["Status"] = status_mapping.STATUS_MAP.get(
            storage_resource["status"])

        self.redfish["Links"] = collections.OrderedDict()
        self.redfish["Links"]["ComputerSystems"] = list()
        self.redfish["Links"]["Zones"] = list()

        self.redfish["CompositionStatus"]["SharingCapable"] = False
        if storage_resource["category"] == "drives":
            if storage_resource["attributes"]["available"] == "yes":
                composit_state = "Unused"
            else:
                composit_state = "Composed"
            self._fill_link_members(drive_index_trees, zone_ids)
        else:
            isShareable = storage_resource["isShareable"]
            if server_profiles:
                if isShareable:
                    composit_state = "ComposedAndAvailable"
                else:
                    composit_state = "Composed"
            else:
                composit_state = "Unused"
            if isShareable:
                self.redfish["CompositionStatus"]["SharingCapable"] = True
            self._fill_volume_link_members(zone_ids, server_profiles)

        self.redfish["CompositionStatus"]["CompositionState"] = composit_state

        self.redfish["Storage"] = [
            {
                "@odata.id": self.redfish["@odata.id"] + "/Storage/1"
            }
        ]

        self._validate()

    def _fill_volume_link_members(self, zone_ids, sp_list):
        for zone_id in zone_ids:
            zone_dict = {
                "@odata.id": ZoneCollection.BASE_URI + "/" + zone_id
            }
            self.redfish["Links"]["Zones"].append(zone_dict)

        for system in sp_list:
            system_dict = {
                "@odata.id": ComputerSystem.BASE_URI + "/" +
                system.split("/")[-1]
            }
            self.redfish["Links"]["ComputerSystems"].append(system_dict)

    def _fill_link_members(self, drive_index_trees, zone_ids):
        sp_uuid = self._get_server_profile_uuid(drive_index_trees)
        if sp_uuid:
            system_dict = {
                "@odata.id": ComputerSystem.BASE_URI + "/" + sp_uuid
            }
            self.redfish["Links"]["ComputerSystems"].append(system_dict)

        current_encl_id = self._get_enclosure_uuid(drive_index_trees)
        if current_encl_id:
            for zone_id in zone_ids:
                _, encl_id = ZoneService\
                    .split_zone_id_to_spt_uuid_and_enclosure_id(zone_id)

                if encl_id == current_encl_id:
                    zone_dict = {
                        "@odata.id": ZoneCollection.BASE_URI + "/" + zone_id
                    }
                    self.redfish["Links"]["Zones"].append(zone_dict)

    def _get_server_profile_uuid(self, drive_index_trees):
        try:
            bay = drive_index_trees["parents"]["DRIVE_BAY_TO_DRIVE_ASSOC"][0]
            sas_jbod = \
                bay["parents"]["SAS_LOGICAL_JBOD_TO_DRIVEBAYS_ASSOCIATION"][0]
            server_profile = \
                sas_jbod["parents"]["SERVERPROFILE_TO_SLJBOD_ASSOCIATION"][0]
            server_profile_uuid = \
                server_profile["resource"]["uri"].split("/")[-1]
        except KeyError as e:
            logging.debug("The key {} was not found inside "
                          "'drive index trees dict' from the Oneview when "
                          "trying get the server profile uuid"
                          .format(e.args[0]))
            server_profile_uuid = None

        return server_profile_uuid

    def _get_enclosure_uuid(self, drive_index_trees):
        try:
            bay = drive_index_trees["parents"]["DRIVE_BAY_TO_DRIVE_ASSOC"][0]
            drive_encl = \
                bay["parents"]["DRIVE_ENCLOSURE_TO_DRIVE_BAY_ASSOC"][0]
            enclosure = \
                drive_encl["parents"]["ENCLOSURE_TO_BLADE"][0]
            enclosure_id = \
                enclosure["resource"]["attributes"]["uuid"]
        except KeyError as e:
            logging.debug("The key {} was not found inside "
                          "'drive index trees dict' from the Oneview when "
                          "trying get the enclosure uuid"
                          .format(e.args[0]))
            enclosure_id = None

        return enclosure_id



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import logging

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import Response
from flask_api import status
from hpOneView.exceptions import HPOneViewException

# Own libs
from oneview_redfish_toolkit.api.manager import Manager
from oneview_redfish_toolkit import client_session
from oneview_redfish_toolkit import multiple_oneview
from oneview_redfish_toolkit.services.manager_service import \
    get_oneview_ip_by_manager_uuid

manager = Blueprint("manager", __name__)


@manager.route("/redfish/v1/Managers/<uuid>", methods=["GET"])
def get_managers(uuid):
    """Get the Redfish Managers.

        Get method to return Managers JSON when
        /redfish/v1/Managers/id is requested.

        Returns:
            JSON: JSON with Managers info.
    """
    try:
        state_url = "/controller-state.json"
        ov_health_status_url = "/rest/appliance/health-status"

        ov_ip = get_oneview_ip_by_manager_uuid(uuid)
        if not ov_ip:
            abort(status.HTTP_404_NOT_FOUND,
                  "Manager with id {} was not found".format(uuid))

        ov_client = client_session.get_oneview_client(ov_ip)

        ov_appliance_info = multiple_oneview.execute_query_ov_client(
            ov_client, "appliance_node_information", "get_version"
        )
        ov_appliance_state = multiple_oneview.execute_query_ov_client(
            ov_client, "connection", "get", state_url
        )
        ov_appliance_health_status = multiple_oneview.execute_query_ov_client(
            ov_client, "connection", "get", ov_health_status_url
        )

        manager = Manager(ov_appliance_info,
                          ov_appliance_state,
                          ov_appliance_health_status
                          )

        json_str = manager.serialize()

        response = Response(
            response=json_str,
            status=status.HTTP_200_OK,
            mimetype="application/json")
        return response
    except HPOneViewException as e:
        # In case of error log exception and abort
        logging.exception(e)
        abort(status.HTTP_404_NOT_FOUND, "Manager not found")



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class EthernetInterfaceCollection(RedfishJsonValidator):
    """Creates a Ethernet Interface Collection Redfish dict

        Populates self.redfish with some hardcoded EthernetInterfaceCollection
        values and Oneview Server Profile information
    """

    SCHEMA_NAME = 'EthernetInterfaceCollection'

    def __init__(self, server_profile):
        """EthernetInterfaceCollection constructor

            Populates self.redfish and calls _validate()

            Args:
                server_profile: a server profile dict from OneView
        """
        super().__init__(self.SCHEMA_NAME)

        odata_uri = ComputerSystem.BASE_URI + "/" + \
            server_profile["uuid"] + "/EthernetInterfaces"

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = "Ethernet Interface Collection"

        connections = server_profile["connectionSettings"]["connections"]
        ethernet_list = []
        for conn in connections:
            if conn["functionType"] == "Ethernet":
                item_redfish = {"@odata.id": odata_uri + "/" + str(conn["id"])}
                ethernet_list.append(item_redfish)

        self.redfish["Members@odata.count"] = len(ethernet_list)
        self.redfish["Members"] = ethernet_list
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#EthernetInterfaceCollection." \
            "EthernetInterfaceCollection"
        self.redfish["@odata.id"] = odata_uri

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Own libs
from oneview_redfish_toolkit import multiple_oneview


def get_manager_uuid(resource_id):
    map_resources = multiple_oneview.get_map_resources()
    ov_ip = map_resources.get(resource_id)

    map_appliances = multiple_oneview.get_map_appliances()
    manager_uuid = map_appliances.get(ov_ip)

    return manager_uuid


def get_oneview_ip_by_manager_uuid(uuid):
    map_appliances = multiple_oneview.get_map_appliances()
    for ov_ip, appliance_uuid in map_appliances.items():
        if appliance_uuid == uuid:
            return ov_ip

    return None



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import collections
from oneview_redfish_toolkit.api.resource_block import ResourceBlock
import oneview_redfish_toolkit.api.status_mapping as status_mapping
from oneview_redfish_toolkit.services.zone_service import ZoneService

STATE_TO_STATUS_MAPPING = {
    "NoProfileApplied": "Unused",
    "ApplyingProfile": "Composing",
    "ProfileApplied": "Composed",
    "ProfileError": "Failed"
}


class ServerHardwareResourceBlock(ResourceBlock):
    """Creates a ResourceBlock Redfish dict for Server Hardware

        Populates self.redfish with some hardcoded ResourceBlock
        values and with Server Hardware data retrieved from OneView.
    """

    def __init__(self, uuid, server_hardware, zone_ids):
        """ServerHardwareResourceBlock constructor

            Populates self.redfish with the contents of server hardware
            and server profile template dicts.

            Args:
                uuid: server hardware UUID
                server_hardware: ServerHardware dict from OneView
                zone_ids: list of Zone ids
        """
        super().__init__(uuid, server_hardware)

        self.server_hardware = server_hardware
        self.zone_ids = zone_ids

        self.redfish["ResourceBlockType"] = ["ComputerSystem"]

        self.redfish["CompositionStatus"]["SharingCapable"] = False

        composition_state = self._get_composition_state()
        self.redfish["CompositionStatus"]["CompositionState"] = \
            composition_state
        self.redfish["Status"] = collections.OrderedDict()
        state, health = status_mapping.\
            get_redfish_server_hardware_status_struct(server_hardware)
        self.redfish["Status"]["State"] = state
        self.redfish["Status"]["Health"] = health

        self._fill_computer_system()
        self._fill_links()

        self._validate()

    def _get_composition_state(self):
        composition_state = status_mapping.\
            get_redfish_composition_state(self.server_hardware)

        if not composition_state:
            composition_state = self._get_server_profile_state()

        return composition_state

    def _get_server_profile_state(self):
        server_profile_uri = self.server_hardware["serverProfileUri"]

        if server_profile_uri:
            return status_mapping.COMPOSITION_STATE["ProfileApplied"]
        else:
            return status_mapping.COMPOSITION_STATE["NoProfileApplied"]

    def _fill_computer_system(self):
        self.redfish["ComputerSystems"] = list()

        computer_system = dict()
        computer_system["@odata.id"] = \
            self.BASE_URI + "/" + self.uuid + "/Systems/1"

        self.redfish["ComputerSystems"].append(computer_system)

    def _fill_links(self):
        self.redfish["Links"] = dict()

        chassi = dict()
        chassi["@odata.id"] = "/redfish/v1/Chassis/" + self.uuid
        self.redfish["Links"]["Chassis"] = list()
        self.redfish["Links"]["Chassis"].append(chassi)

        if self.server_hardware["serverProfileUri"]:
            sp_id = self.server_hardware["serverProfileUri"].split("/")[-1]

            system = dict()
            system["@odata.id"] = "/redfish/v1/Systems/" + sp_id
            self.redfish["Links"]["ComputerSystems"] = list()
            self.redfish["Links"]["ComputerSystems"].append(system)

        if self.zone_ids:
            self.redfish["Links"]["Zones"] = list()
            location_id = self.server_hardware["locationUri"].split("/")[-1]

            for zone_id in self.zone_ids:
                _, encl_id = ZoneService\
                    .split_zone_id_to_spt_uuid_and_enclosure_id(zone_id)

                zone = dict()
                if (encl_id and encl_id == location_id) or not encl_id:
                    zone["@odata.id"] = \
                        "/redfish/v1/CompositionService/ResourceZones/" \
                        + zone_id

                    self.redfish["Links"]["Zones"].append(zone)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import g
from flask import request
from flask import Response
from flask_api import status

# Own libs
from oneview_redfish_toolkit.api.blade_chassis import BladeChassis
from oneview_redfish_toolkit.api.enclosure_chassis import EnclosureChassis
from oneview_redfish_toolkit.api.rack_chassis import RackChassis
from oneview_redfish_toolkit.api.util.power_option import OneViewPowerOption
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit.services.manager_service import \
    get_manager_uuid

chassis = Blueprint("chassis", __name__)


@chassis.route("/redfish/v1/Chassis/<uuid>", methods=["GET"])
def get_chassis(uuid):
    """Get the Redfish Chassis.

        Get method to return Chassis JSON when
        /redfish/v1/Chassis/id is requested.

        Returns:
            JSON: JSON with Chassis.
    """
    category = ''
    cached_category = category_resource.get_category_by_resource_id(uuid)

    if cached_category:
        category = cached_category.resource.replace('_', '-')
    else:
        resource_index = g.oneview_client.index_resources.get_all(
            filter='uuid=' + uuid
        )
        if not resource_index:
            abort(status.HTTP_404_NOT_FOUND,
                  "Chassis {} not found".format(uuid))

        category = resource_index[0]["category"]

    manager_uuid = get_manager_uuid(uuid)

    if category == 'server-hardware':
        server_hardware = g.oneview_client.server_hardware.get_by_id(uuid).data
        etag = server_hardware['eTag']
        ch = BladeChassis(server_hardware, manager_uuid)
    elif category == 'enclosures':
        enclosure = g.oneview_client.enclosures.get_by_id(uuid).data
        etag = enclosure['eTag']

        enclosure_environment_config = g.oneview_client.enclosures. \
            get_environmental_configuration(uuid)
        ch = EnclosureChassis(
            enclosure,
            enclosure_environment_config,
            manager_uuid
        )
    elif category == 'racks':
        racks = g.oneview_client.racks.get(uuid)
        etag = racks['eTag']
        ch = RackChassis(racks)

    return ResponseBuilder.success(ch, {"ETag": "W/" + etag})


@chassis.route("/redfish/v1/Chassis/<uuid>/"
               "Actions/Chassis.Reset", methods=["POST"])
def change_server_hardware_power_state(uuid):
    """Change the Oneview power state for a specific Server hardware.

        Return ResetType Chassis redfish JSON for a
        given chassis UUID.
        Logs exception of any error and return abort.

        Returns:
            JSON: Redfish JSON with Chassis ResetType.

        Exceptions:
            q: When some OneView resource was not found.
            return abort(404)

            Exception: Unexpected error.
            return abort(500)
    """

    try:
        reset_type = request.get_json()["ResetType"]
    except KeyError:
        invalid_key = list(request.get_json())[0]  # gets invalid key name
        abort(status.HTTP_400_BAD_REQUEST,
              "Invalid JSON key: {}".format(invalid_key))

    resource_index = g.oneview_client.index_resources.get_all(
        filter='uuid=' + uuid
    )
    if not resource_index:
        abort(status.HTTP_404_NOT_FOUND,
              "Chassis {} not found".format(uuid))

    category = resource_index[0]["category"]
    if category == 'server-hardware':
        # Gets ServerHardware for given UUID
        sh = g.oneview_client.server_hardware.get_by_id(uuid).data

        # Gets power configuration based on OneView pattern
        oneview_power_configuration = \
            OneViewPowerOption.get_oneview_power_configuration(
                sh, reset_type
            )

        # Changes the ServerHardware power state
        g.oneview_client.server_hardware.update_power_state(
            oneview_power_configuration, uuid)

        return Response(
            response='{"ResetType": "%s"}' % reset_type,
            status=status.HTTP_200_OK,
            mimetype='application/json')



# -*- coding: utf-8 -*-

# Copyright (2017) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.chassis import Chassis


class RackChassis(Chassis):
    """Creates a Rack Chassis Redfish dict

         Populates self.redfish with some hardcoded Rack Chassis
         values and with the response of OneView Rack.
    """

    def __init__(self, rack):
        """Rack Chassis constructor

        Populates self.redfish with hardcoded Rack Chassis
        values and with the response of OneView rack.

        Args:
            rack: An object containing the Oneview rack
            to create the Redfish JSON.
        """

        super().__init__(rack)

        self.redfish["ChassisType"] = "Rack"
        self.redfish["Model"] = rack["model"]
        self.redfish["Links"]["Contains"] = list()
        for enc in rack["rackMounts"]:
            self.redfish["Links"]["Contains"].append({
                "@odata.id": '/redfish/v1/Chassis/' +
                enc['mountUri'].split('/')[-1]
            })

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import g
from flask_api import status

# Own libs
from oneview_redfish_toolkit.api.processor import Processor
from oneview_redfish_toolkit.api.resource_block_collection \
    import ResourceBlockCollection
from oneview_redfish_toolkit.blueprints.util.response_builder \
    import ResponseBuilder

processor = Blueprint("processor", __name__)


@processor.route(
    ResourceBlockCollection.BASE_URI + "/<uuid>/Systems/1/Processors/<id>",
    methods=["GET"])
def get_processor(uuid, id):
    """Get the Redfish Resource Block System Processor.

        Get method to return Resource Block System Processor JSON.

        Returns:
            JSON: JSON with Resource Block System Processor info.
    """

    try:
        processor_id = int(id)
    except ValueError:
        abort(status.HTTP_400_BAD_REQUEST,
              "Cannot convert processor_id {} to integer".format(id))

    server_hardware = g.oneview_client.server_hardware.get_by_id(uuid).data
    processor_count = server_hardware["processorCount"]

    if processor_id < 1 or processor_id > processor_count:
        abort(status.HTTP_404_NOT_FOUND,
              "Invalid processor identifier {}".format(processor_id))

    processor = Processor(server_hardware, str(processor_id))

    return ResponseBuilder.success(
        processor, {"ETag": "W/" + server_hardware["eTag"]})



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class CapabilitiesObject(RedfishJsonValidator):
    """Creates a CapabilitiesObject Redfish dict

        Populates self.redfish with some hardcoded Capability
        values and with the response of OneView.
    """

    SCHEMA_NAME = 'ComputerSystem'
    BASE_URI = '/redfish/v1/Systems'

    def __init__(self, profile_template):
        """Capability constructor

            Populates self.redfish with some hardcoded Capability
            values and with the response of OneView.

            Args:
                profile_template: Oneview's Server profile templates dict
        """
        super().__init__(self.SCHEMA_NAME)

        self.profile_template = profile_template
        uuid = profile_template["uri"].split("/")[-1]

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = uuid
        self.redfish["Name"] = profile_template["name"]

        self.redfish["Description@Redfish.OptionalOnCreate"] = True
        self.redfish["Description@Redfish.UpdatableAfterCreate"] = True
        self.redfish["Id@Redfish.RequiredOnCreate"] = True
        self.redfish["Id@Redfish.AllowableValues"] = [uuid]
        self.redfish["Name@Redfish.RequiredOnCreate"] = True
        self.redfish["Name@Redfish.SetOnlyOnCreate"] = True
        self.redfish["Links@Redfish.RequiredOnCreate"] = True

        links = dict()
        links["ResourceBlocks@Redfish.RequiredOnCreate"] = True
        links["ResourceBlocks@Redfish.UpdatableAfterCreate"] = True
        self.redfish["Links"] = links

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#ComputerSystem.ComputerSystem"
        self.redfish["@odata.id"] = "{}/{}".format(
            self.BASE_URI, uuid)

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import logging

# 3rd party libs
from flask import abort
from flask import Blueprint
from flask import Response
from flask_api import status

# own libs
from oneview_redfish_toolkit.api.composition_service \
    import CompositionService


composition_service = Blueprint("composition_service", __name__)


@composition_service.route("/redfish/v1/CompositionService/", methods=["GET"])
def get_composition_service():
    """Get the Redfish Composition Service.

        Get method to return Composition Service JSON when
        /redfish/v1/CompositionService is requested.

        Returns:
            JSON: Redfish JSON with CompositionService.
    """
    try:
        # Build Composition Service object and validates it
        cs = CompositionService()

        # Build redfish json
        json = cs.serialize()

        return Response(
            response=json,
            status=status.HTTP_200_OK,
            mimetype="application/json")
    except Exception as e:
        # In case of error print exception and abort
        logging.exception(e)
        return abort(status.HTTP_500_INTERNAL_SERVER_ERROR)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
from collections import OrderedDict
import logging
import logging.config
import threading
from threading import Lock
import time
import uuid

# 3rd party libs
from flask import abort
from flask import request
from flask_api import status
from hpOneView import HPOneViewException

# Modules own libs
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import connection
from oneview_redfish_toolkit import multiple_oneview


# Globals vars:
#   globals()['map_clients']


GC_FREQUENCY_IN_SEC = 12 * 60 * 60

lock = Lock()


def _get_map_clients():
    return globals()['map_clients']


def init_map_clients():
    globals()['map_clients'] = OrderedDict()


def init_gc_for_expired_sessions():
    gc_thread = threading.Thread(target=_gc_for_expired_sessions, daemon=True)
    gc_thread.start()


def _gc_for_expired_sessions():
    while True:
        time.sleep(GC_FREQUENCY_IN_SEC)
        logging.debug('Verifying expired sessions...')

        tokens_to_remove = []
        map_items = _get_map_clients().items()
        for token, dict_by_token in map_items:
            ov_clients = iter(dict_by_token['client_ov_by_ip'].values())
            ov_client = next(ov_clients)  # ov_clients can be in any order
            ov_session_id = ov_client.connection.get_session_id()
            try:
                # request that does not increment the life of Oneview session
                resp, body = ov_client.connection.do_http(
                    'GET',
                    '/rest/sessions/',
                    '',
                    {'Session-Id': ov_session_id}
                )

                # when is not a success
                if resp.status not in range(200, 300):
                    if resp.status == 404:
                        tokens_to_remove.append(token)
                    else:
                        logging.error('Unexpected response with status {} '
                                      'of Oneview sessions endpoint: {}'
                                      .format(resp.status, body))

            except Exception as e:
                logging.exception('Unexpected error: {}'.format(e))

        for token in tokens_to_remove:
            clear_session_by_token(token)


def _set_new_client_by_token(redfish_token, client_ov_by_ip):
    with lock:
        globals()['map_clients'][redfish_token] = {
            'client_ov_by_ip': client_ov_by_ip,
            'session_id': str(uuid.uuid4())
        }


def get_session_id_by_token(token):
    session_dict = _get_map_clients().get(token)
    if session_dict:
        return session_dict['session_id']

    return None


def _set_new_clients_by_ip(ov_clients_by_ip):
    with lock:
        globals()['map_clients'] = ov_clients_by_ip


def get_session_ids():
    session_map_items = _get_map_clients().items()
    return [sess_dict['session_id'] for _, sess_dict in session_map_items]


def clear_session_by_token(token):
    with lock:
        if token in _get_map_clients():
            del _get_map_clients()[token]


def login(username, password):
    try:
        clients_ov_by_ip = dict()
        tokens = []

        for ip in config.get_oneview_multiple_ips():
            oneview_client = connection.new_oneview_client(ip, username,
                                                           password)
            tokens.append(oneview_client.connection.get_session_id())

            clients_ov_by_ip[ip] = oneview_client
            _set_manager_into_appliances_map(ip, oneview_client)

        redfish_token = tokens[0]

        _set_new_client_by_token(redfish_token, clients_ov_by_ip)

        return redfish_token, get_session_id_by_token(redfish_token)
    except HPOneViewException as e:
        logging.exception('Unauthorized error: {}'.format(e))
        raise e


def login_conf_mode():
    try:
        clients_ov_by_ip = dict()

        for ip in config.get_oneview_multiple_ips():
            oneview_client = connection.new_oneview_client(ip)
            clients_ov_by_ip[ip] = oneview_client

            _set_manager_into_appliances_map(ip, oneview_client)

        _set_new_clients_by_ip(clients_ov_by_ip)
    except HPOneViewException as e:
        logging.exception('Unauthorized error: {}'.format(e))
        raise e


def check_authentication(rf_token):
    if rf_token not in _get_map_clients():
        msg = 'Unauthorized error for redfish token: {}'.format(rf_token)
        logging.exception(msg)
        abort(status.HTTP_401_UNAUTHORIZED, msg)


def _get_oneview_client_by_token(ip_oneview, token=None):
    try:
        if token:
            rf_token = token
        else:
            rf_token = request.headers.get('x-auth-token')
        client_ov_by_ip = _get_map_clients()[rf_token]['client_ov_by_ip']
        oneview_client = client_ov_by_ip[ip_oneview]
        return oneview_client
    except KeyError:
        msg = 'Unauthorized error for redfish token {}' \
            .format(rf_token)
        logging.exception(msg)
        abort(status.HTTP_401_UNAUTHORIZED, msg)


def _get_oneview_client_by_ip(ip_oneview):
    oneview_client = _get_map_clients()[ip_oneview]
    return oneview_client


def get_oneview_client(ip_oneview, token=None):
    if config.auth_mode_is_session():
        return _get_oneview_client_by_token(ip_oneview, token)

    if config.auth_mode_is_conf():
        return _get_oneview_client_by_ip(ip_oneview)


def _set_manager_into_appliances_map(ip, oneview_client):
    manager = oneview_client.appliance_node_information.get_version()
    multiple_oneview.set_map_appliances_entry(ip, manager["uuid"])



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

STATUS_MAP = {
    "OK": {
        "State": "Enabled",
        "Health": "OK"
    },
    "Disabled": {
        "State": "Disabled",
        "Health": "Warning"
    },
    "Warning": {
        "State": "Enabled",
        "Health": "Warning"
    },
    "Critical": {
        "State": "Enabled",
        "Health": "Critical"
    },
    "Unknown": {
        "State": "Absent",
        "Health": None
    }
}

HEALTH_STATE = {
    "Critical": "Critical",
    "OK": "OK",
    "Unknown": "Warning",
    "Disabled": "Warning",
    "Warning": "Warning",
}

MANAGER_HEALTH_STATE = {
    "INFO": "OK",
    "WARNING": "Warning",
    "CRITICAL": "Critical",
}

COMPOSITION_STATE = {
    "NoProfileApplied": "Unused",
    "ApplyingProfile": "Composing",
    "ProfileApplied": "Composed",
    "ProfileError": "Failed",
    "RemovingProfile": "Composed"
}


SERVER_HARDWARE_STATE_TO_REDFISH_STATE = {
    "NoProfileApplied": "Enabled",
    "Monitored": "Enabled",
    "ProfileApplied": "Enabled",
    "Unsupported": "UnavailableOffline",
    "Unknown": "Absent",
    "RemoveFailed": "StandbyOffline",
    "ProfileError": "StandbyOffline",
    "Unmanaged": "StandbyOffline",
    "Removing": "Updating",
    "ApplyingProfile": "Updating",
    "RemovingProfile": "Updating",
    "UpdatingFirmware": "Updating",
    "Adding": "Updating",
    "Removed": "Disabled"
}

SERVER_PROFILE_STATE_TO_REDFISH_STATE = {
    "Normal": "Enabled",
    "Creating": "Updating",
    "Updating": "Updating",
    "Deleting": "Updating",
    "CreateFailed": "StandbyOffline",
    "UpdateFailed": "StandbyOffline",
    "DeleteFailed": "StandbyOffline"
}

APPLIANCE_STATE_TO_REDFISH_STATE = {
    "OK": "Enabled",
    "UPGRADE": "Updating",
    "RESTORE": "Updating",
    "FACTORY_RESET": "Updating",
    "FAILOVER": "Updating",
    "CONFIGURATION_ERROR": "StandbyOffline",
    "RESTORE_NO_RESET_ERROR": "StandbyOffline",
    "NETWORK_ERROR": "StandbyOffline",
    "INCOMPATIBLE_MODEL_ERROR": "UnavailableOffline",
    "STOPPED": "UnavailableOffline",
    "ERROR": "UnavailableOffline",
    "STARTING": "Starting",
}

CRITICALITY_STATUS = {
    "OK": 1,
    "Warning": 2,
    "Critical": 3
}
RAID_LEVEL = {
    "RAID0": "NonRedundant",
    "RAID1": "Mirrored",
    "RAID5": "StripedWithParity",
    "RAID6": "StripedWithParity",
    "RAID50": "StripedWithParity",
    "RAID60": "StripedWithParity",
    "RAID10": "SpannedMirrors",
    "RAID10ADM": "SpannedMirrors",
    "RAID1ADM": "SpannedMirrors"
}


def get_redfish_server_hardware_status_struct(resource):
    sh_state = SERVER_HARDWARE_STATE_TO_REDFISH_STATE.get(
        resource["state"])
    health_status = HEALTH_STATE.get(resource["status"])

    return sh_state, health_status


def get_redfish_server_profile_state(resource):
    sp_state = SERVER_PROFILE_STATE_TO_REDFISH_STATE.get(
        resource["state"])
    health_status = HEALTH_STATE.get(resource["status"])
    return sp_state, health_status


def get_redfish_composition_state(resource):
    composition_state = COMPOSITION_STATE.get(
        resource["state"])
    return composition_state



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishInvalidAttributeValueException
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class NetworkPort(RedfishJsonValidator):
    """Creates a NetworkPort Redfish dict

        Populates self.redfish with NetworkPort data retrieved from
        OneView
    """

    SCHEMA_NAME = 'NetworkPort'

    def __init__(self, device_id, port_id, server_hardware):
        """NetworkPort constructor

            Populates self.redfish with the contents of server hardware dict
            from Oneview

            Args:
                device_id: ID of the NetworkAdapter
                port_id: ID of the Port.
                server_hardware: Oneview's server hardware dict
        """
        super().__init__(self.SCHEMA_NAME)

        physical_ports = self.get_resource_by_id(
            server_hardware["portMap"]["deviceSlots"], "deviceNumber",
            device_id)["physicalPorts"]

        port = self.get_resource_by_id(physical_ports, "portNumber", port_id)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = port_id
        self.redfish["Name"] = "Physical port {}".format(port_id)
        self.redfish["PhysicalPortNumber"] = port_id
        self.redfish["ActiveLinkTechnology"] = port["type"]
        self.redfish["AssociatedNetworkAddresses"] = list()
        if port["type"] == "Ethernet":
            self.redfish["AssociatedNetworkAddresses"].append(port["mac"])
        elif port["type"] == "FibreChannel":
            self.redfish["AssociatedNetworkAddresses"].append(port["wwn"])
        else:
            raise OneViewRedfishInvalidAttributeValueException(
                "Type not supported"
            )

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#NetworkPort.NetworkPort"
        self.redfish["@odata.id"] = "/redfish/v1/Chassis/" + \
            server_hardware["uuid"] + \
            "/NetworkAdapters/" + device_id + \
            "/NetworkPorts/" + port_id

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class CompositionService(RedfishJsonValidator):
    """Creates a Composition Service Redfish dict

        Populates self.redfish with some hardcoded CompositionService
        values and with information fetched from OneView.
    """

    SCHEMA_NAME = 'CompositionService'

    def __init__(self):
        """CompositionService constructor

            Populates self.redfish with some hardcoded CompositionService
            values and with information fetched from OneView to build the
            ResourceBlocks and ResouceZones.
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = "CompositionService"
        self.redfish["Name"] = "Composition Service"

        self.redfish["Status"] = collections.OrderedDict()
        self.redfish["Status"]["State"] = "Enabled"
        self.redfish["Status"]["Health"] = "OK"

        self.redfish["ServiceEnabled"] = True

        self.redfish["ResourceBlocks"] = collections.OrderedDict()
        self.redfish["ResourceBlocks"]["@odata.id"] = \
            "/redfish/v1/CompositionService/ResourceBlocks"

        self.redfish["ResourceZones"] = collections.OrderedDict()
        self.redfish["ResourceZones"]["@odata.id"] = \
            "/redfish/v1/CompositionService/ResourceZones"

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#CompositionService.CompositionService"
        self.redfish["@odata.id"] = "/redfish/v1/CompositionService"
        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import collections

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class NetworkAdapter(RedfishJsonValidator):
    """Creates a NetworkAdapter Redfish dict

        Populates self.redfish with NetworkAdapter data retrieved from
        OneView
    """

    SCHEMA_NAME = 'NetworkAdapter'

    def __init__(self, device_id, server_hardware):
        """NetworkAdapter constructor

            Populates self.redfish with the contents of server hardware dict
            from Oneview

            Args:
                device_id: The id of the Network Adapter in OneView dict
                server_hardware: Oneview's server hardware dict
        """
        super().__init__(self.SCHEMA_NAME)
        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = device_id

        device_slot = self.get_resource_by_id(
            server_hardware["portMap"]["deviceSlots"],
            "deviceNumber", device_id)

        self.redfish["Name"] = device_slot["deviceName"]

        self.redfish["Controllers"] = list()
        self.redfish["Controllers"].append(collections.OrderedDict())

        # ControllerCapabilities property
        self.redfish["Controllers"][0]["ControllerCapabilities"] = \
            collections.OrderedDict()
        port_count = len(device_slot["physicalPorts"])
        virtual_port_count = 0
        if port_count > 0:
            for i in range(0, port_count):
                virtual_port_count += len(
                    device_slot["physicalPorts"][i]["virtualPorts"])
        self.redfish["Controllers"][0]["ControllerCapabilities"][
            "NetworkPortCount"] = port_count
        self.redfish["Controllers"][0]["ControllerCapabilities"][
            "NetworkDeviceFunctionCount"] = virtual_port_count

        # Removing Links property (since it is causing validation error)
        # and this info is available in the NetworkPorts and
        # NetworkDeviceFunctions links below
        # # Links property
        # self.redfish["Controllers"][0]["Links"] = collections.OrderedDict()
        # self.redfish["Controllers"][0]["Links"]["NetworkPorts"] = list()
        # self.redfish["Controllers"][0]["Links"]["NetworkDeviceFunctions"] = \
        #     list()

        # # Adding NetworkPorts
        # for port in device_slot["physicalPorts"]:
        #     new_port = {
        #         "@odata:id": "/redfish/v1/Chassis/" +
        #         server_hardware["uuid"] +
        #         "/NetworkAdapters/" + device_id + "/NetworkPorts/" +
        #         str(port["portNumber"])}
        #     self.redfish["Controllers"][0]["Links"]["NetworkPorts"].\
        #         append(new_port)

        #     # Adding NetworkDeviceFunction
        #     for virtual_port in port["virtualPorts"]:
        #         network_device_function_id = "_".join((
        #             str(port["portNumber"]),
        #             str(virtual_port["portNumber"]),
        #             virtual_port["portFunction"]))
        #         network_device_function = {
        #             "@odata:id": "/redfish/v1/Chassis/" +
        #             server_hardware["uuid"] + "/NetworkAdapters/" +
        #             device_id + "/NetworkDeviceFunctions/" +
        #             network_device_function_id}
        #         self.redfish["Controllers"][0]["Links"][
        #             "NetworkDeviceFunctions"].append(network_device_function)
        self.redfish["NetworkPorts"] = dict()
        self.redfish["NetworkPorts"]["@odata.id"] = \
            "/redfish/v1/Chassis/" + server_hardware["uuid"] + \
            "/NetworkAdapters/" + device_id + "/NetworkPorts/"

        self.redfish["NetworkDeviceFunctions"] = dict()
        self.redfish["NetworkDeviceFunctions"]["@odata.id"] = \
            "/redfish/v1/Chassis/" + server_hardware["uuid"] + \
            "/NetworkAdapters/" + device_id + "/NetworkDeviceFunctions/"

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#NetworkAdapter.NetworkAdapter"
        self.redfish["@odata.id"] = "/redfish/v1/Chassis/" + \
            server_hardware["uuid"] + "/NetworkAdapters/" + device_id

        self._validate()



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import appdirs
import argparse
import ipaddress
import logging
import os
import sys
import time

# 3rd party libs
import cherrypy
from cherrypy.process.plugins import Daemonizer
from cherrypy.process.plugins import PIDFile
from flask import abort
from flask import Flask
from flask import g
from flask import request
from flask_api import status

# own libs
from hpOneView import HPOneViewException
from paste.translogger import TransLogger

import oneview_redfish_toolkit
from oneview_redfish_toolkit.api.errors import OneViewRedfishException
from oneview_redfish_toolkit.api import scmb
from oneview_redfish_toolkit.api.session_collection import SessionCollection
from oneview_redfish_toolkit.blueprints.chassis import chassis
from oneview_redfish_toolkit.blueprints.chassis_collection \
    import chassis_collection
from oneview_redfish_toolkit.blueprints.composition_service \
    import composition_service
from oneview_redfish_toolkit.blueprints.computer_system import computer_system
from oneview_redfish_toolkit.blueprints.computer_system_collection \
    import computer_system_collection
from oneview_redfish_toolkit.blueprints.ethernet_interface import \
    ethernet_interface
from oneview_redfish_toolkit.blueprints.ethernet_interface_collection import \
    ethernet_interface_collection
from oneview_redfish_toolkit.blueprints.event_service import event_service
from oneview_redfish_toolkit.blueprints.manager import manager
from oneview_redfish_toolkit.blueprints.manager_collection \
    import manager_collection
from oneview_redfish_toolkit.blueprints.metadata import metadata
from oneview_redfish_toolkit.blueprints.network_adapter \
    import network_adapter
from oneview_redfish_toolkit.blueprints.network_adapter_collection \
    import network_adapter_collection
from oneview_redfish_toolkit.blueprints.network_device_function \
    import network_device_function
from oneview_redfish_toolkit.blueprints.network_device_function_collection \
    import network_device_function_collection
from oneview_redfish_toolkit.blueprints.network_interface \
    import network_interface
from oneview_redfish_toolkit.blueprints.network_interface_collection \
    import network_interface_collection
from oneview_redfish_toolkit.blueprints.network_port import network_port
from oneview_redfish_toolkit.blueprints.network_port_collection \
    import network_port_collection
from oneview_redfish_toolkit.blueprints.odata import odata
from oneview_redfish_toolkit.blueprints.processor import processor
from oneview_redfish_toolkit.blueprints.processor_collection \
    import processor_collection
from oneview_redfish_toolkit.blueprints.redfish_base import redfish_base
from oneview_redfish_toolkit.blueprints.resource_block import resource_block
from oneview_redfish_toolkit.blueprints.resource_block_collection \
    import resource_block_collection
from oneview_redfish_toolkit.blueprints.service_root import service_root
from oneview_redfish_toolkit.blueprints.session import session
from oneview_redfish_toolkit.blueprints.session_service import session_service
from oneview_redfish_toolkit.blueprints.storage import storage
from oneview_redfish_toolkit.blueprints.storage_collection \
    import storage_collection
from oneview_redfish_toolkit.blueprints.storage_for_resource_block import \
    storage_for_resource_block
from oneview_redfish_toolkit.blueprints.subscription\
    import add_subscription_from_file
from oneview_redfish_toolkit.blueprints.subscription\
    import subscription
from oneview_redfish_toolkit.blueprints.subscription_collection \
    import subscription_collection
from oneview_redfish_toolkit.blueprints.thermal import thermal
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit.blueprints.vlan_network_interface import \
    vlan_network_interface
from oneview_redfish_toolkit.blueprints.zone import zone
from oneview_redfish_toolkit.blueprints.zone_collection import zone_collection
from oneview_redfish_toolkit import category_resource
from oneview_redfish_toolkit import client_session
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit.config import PERFORMANCE_LOGGER_NAME
from oneview_redfish_toolkit import connection
from oneview_redfish_toolkit import handler_multiple_oneview
from oneview_redfish_toolkit import multiple_oneview
from oneview_redfish_toolkit import util


PID_FILE_NAME = 'toolkit.pid'
CFG_DIR_NAME = 'oneview-redfish-toolkit'


def main(config_file_path, logging_config_file_path,
         is_dev_env=False, is_debug_mode=False):
    # Load config file, schemas and creates a OV connection
    try:
        config.configure_logging(logging_config_file_path)
        config.load_config(config_file_path)
    except Exception as e:
        logging.exception('Failed to load app configuration')
        logging.exception(e)
        exit(1)

    # Check auth mode
    auth_mode = config.get_authentication_mode()

    if auth_mode not in ["conf", "session"]:
        logging.error(
            "Invalid authentication_mode. Please check your conf"
            " file. Valid values are 'conf' or 'session'")

    # Flask application
    app = Flask(__name__)

    app.url_map.strict_slashes = False

    # Register blueprints
    app.register_blueprint(redfish_base, url_prefix="/redfish/")
    app.register_blueprint(service_root, url_prefix='/redfish/v1/')
    app.register_blueprint(event_service)
    app.register_blueprint(session_service)
    app.register_blueprint(chassis_collection)
    app.register_blueprint(computer_system_collection)
    app.register_blueprint(computer_system)
    app.register_blueprint(composition_service)
    app.register_blueprint(chassis)
    app.register_blueprint(ethernet_interface)
    app.register_blueprint(ethernet_interface_collection)
    app.register_blueprint(manager_collection)
    app.register_blueprint(manager)
    app.register_blueprint(metadata)
    app.register_blueprint(odata)
    app.register_blueprint(storage)
    app.register_blueprint(thermal)
    app.register_blueprint(storage_collection)
    app.register_blueprint(network_adapter_collection)
    app.register_blueprint(network_interface_collection)
    app.register_blueprint(network_port_collection)
    app.register_blueprint(network_device_function_collection)
    app.register_blueprint(network_device_function)
    app.register_blueprint(network_interface)
    app.register_blueprint(network_adapter)
    app.register_blueprint(network_port)
    app.register_blueprint(processor)
    app.register_blueprint(processor_collection)
    app.register_blueprint(storage_for_resource_block)
    app.register_blueprint(resource_block_collection)
    app.register_blueprint(resource_block)
    app.register_blueprint(vlan_network_interface)
    app.register_blueprint(zone_collection)
    app.register_blueprint(zone)
    app.register_blueprint(subscription_collection)
    app.register_blueprint(subscription)

    # Init cached data
    client_session.init_map_clients()
    scmb.init_map_scmb_connections()
    client_session.init_gc_for_expired_sessions()
    multiple_oneview.init_map_resources()
    multiple_oneview.init_map_appliances()
    category_resource.init_map_category_resources()

    if auth_mode == "conf":
        client_session.login_conf_mode()
    else:
        app.register_blueprint(session)

    @app.before_request
    def init_performance_data():
        if logging.getLogger().isEnabledFor(logging.DEBUG):
            g.start_time_req = time.time()
            g.elapsed_time_ov = 0

    @app.before_request
    def check_authentication():
        # If authenticating do not check for anything
        if request.url_rule and \
           request.url_rule.rule == SessionCollection.BASE_URI and \
           request.method == "POST":
            return None

        if connection.is_service_root():
            return None

        if config.auth_mode_is_session():
            x_auth_token = request.headers.get('x-auth-token')
            client_session.check_authentication(x_auth_token)

        g.oneview_client = \
            handler_multiple_oneview.MultipleOneViewResource()

    @app.before_request
    def has_odata_version_header():
        """Deny request that specify a different OData-Version than 4.0"""
        odata_version_header = request.headers.get("OData-Version")

        if odata_version_header is None:
            pass
        elif odata_version_header != "4.0":
            abort(status.HTTP_412_PRECONDITION_FAILED,
                  "The request specify a different OData-Version "
                  "header then 4.0. This server also responds "
                  "to requests without the OData-Version header")

    @app.after_request
    def set_odata_version_header(response):
        """Set OData-Version header for all responses"""
        response.headers["OData-Version"] = "4.0"
        return response

    @app.after_request
    def log_performance_data(response):
        if logging.getLogger().isEnabledFor(logging.DEBUG):
            end_time = time.time()
            req_time = end_time - g.start_time_req
            logging.getLogger(PERFORMANCE_LOGGER_NAME).debug(
                "OneView process: " + str(g.elapsed_time_ov))
            logging.getLogger(PERFORMANCE_LOGGER_NAME).debug(
                "Redfish process: " + str(req_time - g.elapsed_time_ov))
            logging.getLogger(PERFORMANCE_LOGGER_NAME).debug(
                "Total process: " + str(req_time))
        return response

    @app.errorhandler(status.HTTP_400_BAD_REQUEST)
    def bad_request(error):
        """Creates a Bad Request Error response"""
        logging.error(error.description)

        return ResponseBuilder.error_400(error)

    @app.errorhandler(status.HTTP_401_UNAUTHORIZED)
    def unauthorized_error(error):
        """Creates a Unauthorized Error response"""
        logging.error(error.description)

        return ResponseBuilder.error_401(error)

    @app.errorhandler(status.HTTP_403_FORBIDDEN)
    def forbidden(error):
        """Creates a Forbidden Error response"""
        logging.error(error.description)

        return ResponseBuilder.error_403(error)

    @app.errorhandler(status.HTTP_404_NOT_FOUND)
    def not_found(error):
        """Creates a Not Found Error response"""
        logging.error(error.description)

        return ResponseBuilder.error_404(error)

    @app.errorhandler(status.HTTP_500_INTERNAL_SERVER_ERROR)
    def internal_server_error(error):
        """Creates an Internal Server Error response"""
        logging.error(error)

        return ResponseBuilder.error_500(error)

    @app.errorhandler(status.HTTP_501_NOT_IMPLEMENTED)
    def not_implemented(error):
        """Creates a Not Implemented Error response"""
        logging.error(error.description)

        return ResponseBuilder.error_501(error)

    @app.errorhandler(HPOneViewException)
    def hp_oneview_client_exception(exception):
        logging.exception(exception)
        response = ResponseBuilder.error_by_hp_oneview_exception(exception)

        # checking if session has expired on Oneview
        if config.auth_mode_is_session() and \
                response.status_code == status.HTTP_401_UNAUTHORIZED:
            token = request.headers.get('x-auth-token')
            client_session.clear_session_by_token(token)

        return response

    @app.errorhandler(OneViewRedfishException)
    def oneview_redfish_exception(exception):
        logging.exception(exception)

        return ResponseBuilder.oneview_redfish_exception(exception)

    logging.info("RedfishVersion : " + oneview_redfish_toolkit.version())

    if config.auth_mode_is_conf() and add_subscription_from_file():
        scmb.init_event_service()

    app_config = config.get_config()

    try:
        host = app_config["redfish"]["redfish_host"]

        # Gets the correct IP type based on the string
        ipaddress.ip_address(host)
    except ValueError:
        logging.error("Informed IP is not valid. Check the "
                      "variable 'redfish_host' on your config file.")
        exit(1)

    try:
        port = int(app_config["redfish"]["redfish_port"])
    except Exception:
        logging.exception(
            "Port must be an integer number between 1 and 65536.")
        exit(1)
    # Checking port range
    if port < 1 or port > 65536:
        logging.error("Port must be an integer number between 1 and 65536.")
        exit(1)

    if app_config["ssl"]["SSLType"] in ("self-signed", "adhoc"):
        logging.warning("Server is starting with a self-signed certificate.")
    if app_config["ssl"]["SSLType"] == "disabled":
        logging.warning(
            "Server is starting in HTTP mode. This is an insecure mode. "
            "Running the server with HTTPS enabled is highly recommended.")

    ssl_type = app_config["ssl"]["SSLType"]
    # Check SSLType:
    if ssl_type not in ('disabled', 'adhoc', 'certs', 'self-signed'):
        logging.error(
            "Invalid SSL type: {}. Must be one of: disabled, adhoc, "
            "self-signed or certs".
            format(ssl_type))
        exit(1)

    if ssl_type == 'disabled':
        app.run(host=host, port=port, debug=is_debug_mode)
    elif ssl_type == 'adhoc':
        app.run(host=host, port=port, debug=is_debug_mode, ssl_context="adhoc")
    else:
        # We should use certs file provided by the user
        ssl_cert_file = app_config["ssl"]["SSLCertFile"]
        ssl_key_file = app_config["ssl"]["SSLKeyFile"]

        # Generating cert files if they don't exists
        if ssl_type == "self-signed":
            if not os.path.exists(ssl_cert_file) and not \
                    os.path.exists(ssl_key_file):
                logging.warning("Generating self-signed certs")
                # Generate certificates
                file_name = "self-signed"
                util.generate_certificate(
                    os.path.dirname(ssl_cert_file), file_name, 2048)

                if ssl_cert_file == "" or ssl_key_file == "":
                    ssl_cert_file = file_name + ".crt"
                    ssl_key_file = file_name + ".key"
                    logging.warning("The paths to the certs files were "
                                    "not found in the redfish.conf file. "
                                    "Using generated files %s and %s" %
                                    (ssl_cert_file, ssl_key_file))
            else:
                logging.warning("Using existing self-signed certs")
        elif ssl_cert_file == "" or ssl_key_file == "":
            logging.error(
                "SSL type: is 'cert' but one of the files are missing on"
                "the config file. SSLCertFile: {}, SSLKeyFile: {}.".format(
                    ssl_cert_file, ssl_key_file))

        if is_dev_env and is_debug_mode:
            ssl_context = (ssl_cert_file, ssl_key_file)
            app.run(host=host, port=port, debug=is_debug_mode,
                    ssl_context=ssl_context)
        else:
            is_service_up = check_if_pid_exists()
            if is_service_up:
                print('Application is already running')
                sys.exit(1)
            else:
                start_cherrypy(app,
                               host=host,
                               port=port,
                               ssl_cert_file=ssl_cert_file,
                               ssl_key_file=ssl_key_file,
                               is_dev_env=is_dev_env)


def check_if_pid_exists():
    is_pid_alive = False
    cfg_dir = appdirs.user_config_dir(CFG_DIR_NAME)
    pid_file_path = os.path.join(cfg_dir, PID_FILE_NAME)
    if not os.path.isfile(pid_file_path):
        is_pid_alive = False
    else:
        pid_file = open(pid_file_path, 'r')
        pid = pid_file.read()
        # Check if process is already running or not
        try:
            # Sending signal 0 to a pid will raise an OSError exception
            # if the pid is not running.
            os.kill(int(pid), 0)
        except OSError:
            is_pid_alive = False
        else:
            is_pid_alive = True
    return is_pid_alive


def start_cherrypy(app,
                   host=None,
                   port=None,
                   ssl_cert_file=None,
                   ssl_key_file=None,
                   is_dev_env=None):

    if not is_dev_env:
        cherrypy.config.update({'environment': 'production'})

    cherrypy.config.update(config.get_cherrypy_config())

    cherrypy.config.update({
        'log.screen': False,
        'server.socket_port': port,
        'server.socket_host': host,
        'server.ssl_certificate': ssl_cert_file,
        'server.ssl_private_key': ssl_key_file
    })

    app_logged = TransLogger(app.wsgi_app, setup_console_handler=False)
    cherrypy.tree.graft(app_logged, '/')

    if not is_dev_env:
        Daemonizer(cherrypy.engine).subscribe()
        PIDFile(cherrypy.engine, os.path.join(util.get_user_directory(),
                                              PID_FILE_NAME)).subscribe()

    cherrypy.engine.start()
    cherrypy.engine.block()


if __name__ == '__main__':

    parser = argparse.ArgumentParser(description='Arguments parser')
    parser.add_argument('--config', type=str,
                        help='A required path to config file')
    parser.add_argument('--log-config', type=str,
                        help='A required path to logging config file')
    parser.add_argument('--dev', type=bool, nargs='?',
                        default=False, const=True,
                        help='Optional value to tell the application to run '
                             'in development mode.')
    parser.add_argument('--debug', type=bool, nargs='?',
                        default=False, const=True,
                        help='Optional value to tell the application to run '
                             'in debug mode, this option only is valid when '
                             'the development mode is set too, otherwise '
                             'it is ignored.')
    args = parser.parse_args()
    main(args.config, args.log_config,
         is_dev_env=args.dev, is_debug_mode=args.debug)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
from oneview_redfish_toolkit import config
from oneview_redfish_toolkit import util


class EventService(RedfishJsonValidator):
    """Creates a Event Service dict

        Populates self.redfish with EventService values.
    """

    SCHEMA_NAME = 'EventService'
    BASE_URI = '/redfish/v1/EventService'

    def __init__(self, delivery_retry_attempts, delivery_retry_interval):
        """Constructor

            Populates self.redfish with some hardcoded EventService
            values and values from redfish.conf.
        """

        super().__init__(self.SCHEMA_NAME)

        self._build_common_values(delivery_retry_attempts,
                                  delivery_retry_interval)

        # Enable Event Service only when authentication mode is conf
        if config.auth_mode_is_conf():
            self.redfish["ServiceEnabled"] = True
            self._build_values_when_service_is_enabled()
        else:
            self.redfish["ServiceEnabled"] = False

        self._validate()

    def _build_common_values(self,
                             delivery_retry_attempts,
                             delivery_retry_interval):
        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["@odata.context"] = "/redfish/v1/$metadata" \
                                         "#EventService.EventService"
        self.redfish["@odata.id"] = self.BASE_URI

        self.redfish["Id"] = "EventService"
        self.redfish["Name"] = "Event Service"
        self.redfish["Description"] = "Event Subscription service"

        self.redfish["DeliveryRetryAttempts"] = delivery_retry_attempts
        self.redfish["DeliveryRetryIntervalSeconds"] = delivery_retry_interval

        # TODO(someone) All information below is commented
        # until we decide where to find it
        # self.redfish["Status"] = dict()
        # self.redfish["Status"]["Health"] = "OK"
        # self.redfish["Status"]["HealthRollup"] = "OK"
        # self.redfish["Status"]["State"] = "Enabled"

    def _build_values_when_service_is_enabled(self):
        event_types = sorted(util.get_subscriptions_by_type().keys())

        submit_test_event = dict()
        submit_test_event["EventType@Redfish.AllowableValues"] = event_types
        submit_test_event["target"] = self.BASE_URI + \
            "/Actions/EventService.SubmitTestEvent/"

        self.redfish["Actions"] = dict()
        self.redfish["Actions"]["#EventService.SubmitTestEvent"] = \
            submit_test_event

        self.redfish["Subscriptions"] = dict()
        self.redfish["Subscriptions"]["@odata.id"] = \
            self.BASE_URI + "/EventSubscriptions/"
        self.redfish["EventTypesForSubscription"] = event_types



#!./redfish-venv/bin/python
# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import collections

from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishException
from oneview_redfish_toolkit.api.errors import \
    OneViewRedfishResourceNotFoundException
from oneview_redfish_toolkit.api.redfish_json_validator import \
    RedfishJsonValidator
from oneview_redfish_toolkit import config


class RedfishError(RedfishJsonValidator):
    """Creates a Redfish Error Dict

        Populates self.redfish with errors. Will not validate as there's no
        schema to validate against.

    """

    SCHEMA_NAME = None

    def __init__(self, code, message):
        """Constructor

            Populates self.redfish with error message.
        """

        super().__init__(self.SCHEMA_NAME)
        self.redfish["error"] = collections.OrderedDict()
        # Check if Code is a valid Code Error in the registry
        if code not in config.get_registry_dict()["Base"]["Messages"]:
            raise OneViewRedfishResourceNotFoundException(
                "Registry {} not found.".format(code)
            )

        self.redfish["error"]["code"] = "Base.1.1." + code
        self.redfish["error"]["message"] = message
        self.redfish["error"]["@Message.ExtendedInfo"] = list()

    def add_extended_info(
        self,
        message_id,
        message_args=[],
        related_properties=[]):
        """Adds an item to ExtendedInfo list using values from DMTF registry

            Adds an item to ExtendedInfo list using the values for Message,
            Severity and Resolution from DMTF Base Registry.

            Parameters:
                message_id: Id of the message; oneOf the keys in Redfish
                    Registry Messages
                message_args: List of string to replace markers on Redfish
                    messages. Must have the same length as the number of %
                    signs found in the registry Message field
                related_properties: Properties relates to this e error if
                    necessary

        """
        messages = config.get_registry_dict()["Base"]["Messages"]

        # Verify if message_id exists in registry
        try:
            severity = messages[message_id]["Severity"]
        except Exception:
            raise OneViewRedfishResourceNotFoundException(
                "Message id {} not found.".format(message_id)
            )

        message = messages[message_id]["Message"]

        # Check if numbers of replacements and message_args length match
        replaces = message.count('%')
        replacements = len(message_args)
        if replaces != replacements:
            raise OneViewRedfishException(
                'Message has {} replacements to be made but {} args '
                'where sent'.format(replaces, replacements)
            )

        # Replacing the marks in the message. A better way to do this
        # is welcome.
        for i in range(replaces):
            message = message.replace('%' + str(i + 1), message_args[i])

        # Construct the dict
        extended_info = collections.OrderedDict()
        extended_info["@odata.type"] = "#Message.v1_0_5.Message"
        extended_info["MessageId"] = "Base.1.1." + message_id
        extended_info["Message"] = message
        extended_info["RelatedProperties"] = related_properties
        extended_info["MessageArgs"] = message_args
        extended_info["Severity"] = severity
        extended_info["Resolution"] = messages[message_id]["Resolution"]

        # Append it to the list
        self.redfish["error"]["@Message.ExtendedInfo"].append(extended_info)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

__title__ = 'oneview-redfish-toolkit'

__version__ = '0.4.0'

__copyright__ = '(C) Copyright (2018-2019) Hewlett Packard Enterprise '\
    'Development LP'

__license__ = 'Apache License 2.0'


def get_copyright():
    return __copyright__


def version():
    return __version__



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.resource_block import ResourceBlock
from oneview_redfish_toolkit.api.zone_collection import ZoneCollection


class ServerProfileTemplateResourceBlock(ResourceBlock):
    """Creates a ResourceBlock Redfish dict for Server Profile Template

        Populates self.redfish with some hardcoded ResourceBlock
        values and with Server Profile Template data retrieved from OneView.
    """

    def __init__(self, uuid, server_profile_template, zone_ids):
        """ServerProfileTemplateResourceBlock constructor

            Populates self.redfish with the contents of server profile
            template dict.

            Args:
                uuid: server profile template UUID
                server_profile_template: OneView server profile template
                zone_ids: Ids of Zones that this resource block belongs to
        """
        super().__init__(uuid, server_profile_template)

        self.server_profile_template = server_profile_template

        self.redfish["ResourceBlockType"] = ["Network"]
        self.redfish["CompositionStatus"]["SharingCapable"] = True
        self.redfish["CompositionStatus"]["SharingEnabled"] = True
        self.redfish["CompositionStatus"]["CompositionState"] = "Unused"

        self._fill_ethernet_networks()

        self.redfish["Links"] = dict()
        self.redfish["Links"]["Zones"] = list()

        self._fill_link_members(zone_ids)

    def _fill_link_members(self, zone_ids):
        for zone_id in zone_ids:
            zone = {
                "@odata.id": ZoneCollection.BASE_URI + "/" + zone_id
            }
            self.redfish["Links"]["Zones"].append(zone)

    def _fill_ethernet_networks(self):
        self.redfish["EthernetInterfaces"] = list()

        connSettings = self.server_profile_template["connectionSettings"]

        for conn in connSettings["connections"]:
            if "Ethernet" == conn["functionType"]:
                network = dict()
                network["@odata.id"] = self.BASE_URI + "/" + self.uuid \
                    + "/EthernetInterfaces/" + str(conn["id"])

                self.redfish["EthernetInterfaces"].append(network)

        self.redfish["EthernetInterfaces@odata.count"] = \
            len(self.redfish["EthernetInterfaces"])



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.redfish_json_validator import \
    RedfishJsonValidator
from oneview_redfish_toolkit.api.resource_block_collection import \
    ResourceBlockCollection
from oneview_redfish_toolkit.api import status_mapping


class Drive(RedfishJsonValidator):
    """Creates a StorageDriveCompositionDetails dict

            Populates self.redfish with data retrieved from
            an OneView's Drive
    """

    SCHEMA_NAME = 'Drive'
    METADATA_INFO = "/redfish/v1/$metadata#Drive.Drive"

    def __init__(self, data):
        """StorageDriveCompositionDetails constructor

            Populates self.redfish and validates the result

            Args:
                data: a dict with Redfish's Drive data
        """
        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()

        self.redfish.update(data)

        self.redfish["@odata.context"] = self.__class__.METADATA_INFO

        self._validate()

    @staticmethod
    def build_for_computer_system(drive_id, server_profile, logical_jbod):
        enclosure_id = server_profile["enclosureUri"].split("/")[-1]
        profile_uuid = server_profile["uri"].split("/")[-1]
        odata_id = "{}/{}/Storage/1/Drives/{}"\
            .format(ComputerSystem.BASE_URI, profile_uuid, drive_id)
        media_type = logical_jbod["driveTechnology"]["driveMedia"]
        media_type = None if (media_type == "Unknown") else media_type
        attrs = {
            "Id": str(drive_id),
            "Name": logical_jbod["name"],
            "Status": status_mapping.STATUS_MAP.get(logical_jbod["status"]),
            "CapacityBytes": Drive.get_capacity_in_bytes(
                logical_jbod["maxSizeGB"]),
            "Protocol": logical_jbod["driveTechnology"]["deviceInterface"],
            "MediaType": media_type,
            "Links": {
                "Chassis": {
                    "@odata.id": "/redfish/v1/Chassis/" + enclosure_id
                }
            },
            "@odata.id": odata_id
        }

        return Drive(attrs)

    @staticmethod
    def build_for_resource_block(drive, drive_enclosure):
        enclosure_id = drive_enclosure["enclosureUri"].split("/")[-1]
        drive_uuid = drive["uri"].split("/")[-1]
        ov_drive_attrs = drive["attributes"]
        odata_id = "{}/{}/Storage/1/Drives/1"\
            .format(ResourceBlockCollection.BASE_URI, drive_uuid)
        media_type = ov_drive_attrs["mediaType"]
        media_type = None if (media_type == "Unknown") else media_type
        attrs = {
            "Id": "1",
            "Name": drive["name"],
            "Status": status_mapping.STATUS_MAP.get(drive["status"]),
            "CapacityBytes": Drive.get_capacity_in_bytes(
                ov_drive_attrs["capacityInGB"]),
            "Protocol": ov_drive_attrs["interfaceType"],
            "MediaType": media_type,
            "Links": {
                "Chassis": {
                    "@odata.id": "/redfish/v1/Chassis/" + enclosure_id
                }
            },
            "@odata.id": odata_id
        }

        return Drive(attrs)

    @staticmethod
    def get_capacity_in_bytes(capacity_in_gb):
        size_in_bytes = float(capacity_in_gb) * 1024 * 1024 * 1024
        return int(size_in_bytes)



# -*- coding: utf-8 -*-

# Copyright (2017) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import Blueprint
from flask import Response
from flask_api import status

redfish_base = Blueprint("redfish_base", __name__)


@redfish_base.route("/", methods=["GET"])
def get_redfish_base():
    """Get JSON with Redfish version.

        Returns:
            json: Redfish JSON root.
    """
    return Response(
        response='{"v1": "/redfish/v1/"}',
        status=status.HTTP_200_OK,
        mimetype="application/json")



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# Python libs
import collections

# Modules own libs
from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator
import oneview_redfish_toolkit.api.status_mapping as status_mapping


class Manager(RedfishJsonValidator):
    """Creates a Manager Redfish dict

        Populates self.redfish with some hardcoded Manager
        values and with the response from OneView.
    """

    SCHEMA_NAME = 'Manager'

    def __init__(self, oneview_appliance_info, oneview_appliance_state,
                 oneview_appliance_health_status):
        """Manager constructor

            Populates self.redfish with some hardcoded Manager
            values and with the response from OneView.

            Args:
                oneview_appliance_info: An Oneview's appliance info dict
                oneview_appliance_state: An Oneview's appliance status dict
                oneview_appliance_health_status: An Oneview's appliance
                health state dict
        """

        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = oneview_appliance_info['uuid']
        self.redfish["Description"] = oneview_appliance_info["family"]
        self.redfish["ManagerType"] = "Service"
        self.redfish["FirmwareVersion"] = \
            oneview_appliance_info["softwareVersion"]
        self.redfish["Status"] = collections.OrderedDict()
        state = status_mapping.APPLIANCE_STATE_TO_REDFISH_STATE.\
            get(oneview_appliance_state["state"])
        health = self._get_highest_health_state(
            oneview_appliance_health_status["members"])
        self.redfish["Status"]["State"] = state
        self.redfish["Status"]["Health"] = health
        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#Manager.Manager"
        self.redfish["@odata.id"] = \
            "/redfish/v1/Managers/" + oneview_appliance_info['uuid']

    @staticmethod
    def _get_highest_health_state(health_state_members):
        health_state_result = None
        highest_status = 0

        for member in health_state_members:
            redfish_health_state = status_mapping.MANAGER_HEALTH_STATE. \
                get(member["severity"])
            current_status = \
                status_mapping.CRITICALITY_STATUS[redfish_health_state]

            if current_status > highest_status:
                highest_status = current_status
                health_state_result = redfish_health_state

        return health_state_result



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class VLanNetworkInterfaceCollection(RedfishJsonValidator):
    """Creates a VLanNetworkInterfaceCollection Redfish dict

        Populates self.redfish with network set data retrieved
        from OneView
    """

    SCHEMA_NAME = 'VLanNetworkInterfaceCollection'

    def __init__(self, network_set, endpoint):
        """VLanNetworkInterfaceCollection constructor

            Populates self.redfish with NetworkSet
            data retrieved from OneView.

            Args:
                network_set: Network Set dict from OneView
                endpoint: endpoint uri from original REST
        """

        super().__init__(self.SCHEMA_NAME)

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Name"] = network_set["name"]
        self.redfish["Members@odata.count"] = len(network_set["networkUris"])
        self.redfish["Members"] = list()
        self._set_vlans_to_network_collection(endpoint,
                                              network_set["networkUris"])

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#VLanNetworkInterfaceCollection"
        self.redfish["@odata.id"] = endpoint

        self._validate()

    def _set_vlans_to_network_collection(self, endpoint,
                                         ethernet_networks_uri):
        """Mounts the list of VLans Links

            Populates self.redfish["Members"] with the links
            to all VLanNetworkInterfaces it contains.

            Args:
                ethernet_networks_uri: list of URIs for the
                VLanNetworkInterfaces.
                endpoint: endpoint uri from original REST.
        """

        for ethernet_uri in ethernet_networks_uri:
            vlan = dict()
            vlan["@odata.id"] = endpoint + "/" + ethernet_uri.split("/")[-1]
            self.redfish["Members"].append(vlan)



# -*- coding: utf-8 -*-

# Copyright (2017) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import abort
from flask import Blueprint
from flask import g
from flask_api import status

from oneview_redfish_toolkit.api.network_port_collection \
    import NetworkPortCollection
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder


network_port_collection = Blueprint("network_port_collection", __name__)


@network_port_collection.route(
    "/redfish/v1/Chassis/<server_hardware_uuid>/NetworkAdapters/"
    "<int:device_id>/NetworkPorts/", methods=["GET"])
def get_network_port_collection(server_hardware_uuid, device_id):
    """Get the Redfish Network Port Collection.

        Return NetworkPortCollection Redfish JSON.
    """

    if device_id < 1:
        abort(status.HTTP_500_INTERNAL_SERVER_ERROR)

    server_hardware = g.oneview_client. \
        server_hardware.get(server_hardware_uuid)

    npc = NetworkPortCollection(server_hardware, device_id)

    return ResponseBuilder.success(npc)



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


ONEVIEW_TO_REDFISH_EVENTS = {
    "Created": "ResourceAdded",
    "Updated": "ResourceUpdated",
    "Deleted": "ResourceRemoved",

    # OneView does not have an equivalent change type to StatusChange
    "StatusChange": "StatusChange"
}


class Event(RedfishJsonValidator):
    """Creates a Event Redfish dict

        Populates self.redfish with some hardcoded Event
        values and with the SCMB message received from OneView.
    """

    SCHEMA_NAME = 'Event'

    def __init__(self, oneview_message):
        """Event constructor

        Populates self.redfish with some hardcoded Event
        values and with the SCMB message received from OneView.

            Args:
                oneview_message: A dict with an OneView SCMB message
        """

        super().__init__(self.SCHEMA_NAME)

        category = oneview_message["resource"]["category"]

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["@odata.context"] = "/redfish/v1/$metadata#Event.Event"
        self.redfish["Events@odata.count"] = 1
        self.redfish["Events"] = list()

        if (category == "alerts"):
            self.build_event_from_alert(oneview_message)
        else:
            self.build_event_from_task(oneview_message)

        self._validate()

    def build_event_from_alert(self, oneview_alert):
        associated_resource = oneview_alert["resource"]["associatedResource"]
        event_id = str(os.path.basename(associated_resource["resourceUri"]))

        self.redfish["Id"] = event_id
        self.redfish["Name"] = associated_resource["resourceName"]

        event_record = {}
        event_record["EventTimestamp"] = oneview_alert["timestamp"]
        event_record["EventType"] = "Alert"
        event_record["MessageId"] = "Base.1.1.Success"
        # TODO(svoboda) add link to Redfish resource
        # event_record["OriginOfCondition"] = origin_of_condition

        self.redfish["Events"].append(event_record)

    def build_event_from_task(self, oneview_task):
        event_id = str(os.path.basename(oneview_task["resourceUri"]))

        self.redfish["Id"] = event_id
        self.redfish["Name"] = oneview_task["resource"]["name"]

        event_record = {}
        event_record["EventTimestamp"] = oneview_task["timestamp"]
        event_record["EventType"] = \
            ONEVIEW_TO_REDFISH_EVENTS[oneview_task["changeType"]]
        event_record["MessageId"] = "Base.1.1.Success"
        # TODO(svoboda) add link to Redfish resource
        # event_record["OriginOfCondition"] = origin_of_condition

        self.redfish["Events"].append(event_record)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import abort
from flask import Blueprint
from flask import g
from flask_api import status

from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.ethernet_interface import EthernetInterface
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder

ethernet_interface = Blueprint("ethernet_interface", __name__)


@ethernet_interface.route(
    ComputerSystem.BASE_URI +
    "/<server_profile_uuid>/EthernetInterfaces/<eth_id>",
    methods=["GET"])
def get_ethernet_interface(server_profile_uuid, eth_id):
    """Get the Redfish Ethernet Interface.

    Return EthernetInterface Redfish JSON.
    """
    profile = g.oneview_client.server_profiles.get_by_id(
        server_profile_uuid).data

    connections = profile["connectionSettings"]["connections"]

    connection = None
    for conn in connections:
        if str(conn["id"]) == str(eth_id):
            connection = conn
            break

    if connection is None:
        abort(status.HTTP_404_NOT_FOUND, "EthernetInterface {} not found"
              .format(eth_id))

    network_attrs = g.oneview_client.index_resources\
        .get(connection["networkUri"])

    ethernet = EthernetInterface.build(profile, connection, network_attrs)

    return ResponseBuilder.success(ethernet)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import logging

from flask import abort
from flask import Blueprint
from flask import g
from flask import request
from flask_api import status

from oneview_redfish_toolkit.api.computer_system import ComputerSystem
from oneview_redfish_toolkit.api.resource_block import ResourceBlock
from oneview_redfish_toolkit.api.vlan_network_interface \
    import VLanNetworkInterface
from oneview_redfish_toolkit.api.vlan_network_interface_collection \
    import VLanNetworkInterfaceCollection
from oneview_redfish_toolkit.blueprints.util.response_builder \
    import ResponseBuilder

vlan_network_interface = Blueprint("vlan_network_interface", __name__)


@vlan_network_interface.route(
    ComputerSystem.BASE_URI +
    "/<sp_uuid>/EthernetInterfaces/<connection_id>/VLANs", methods=["GET"])
def get_vlan_network_interface_collection_sp(sp_uuid, connection_id):
    """Get Redfish VLanNetworkInterfaceCollection for the given UUID and ID.

        Return VLanNetworkInterfaceCollection redfish JSON for a given
        ServerProfile UUID and connection ID.

        Returns:
            JSON: Redfish json with VLanNetworkInterfaceCollection.
    """

    server_profile = \
        g.oneview_client.server_profiles.get_by_id(sp_uuid).data

    connection = \
        _get_connection_oneview_resource(server_profile,
                                         connection_id)

    vlan_collection = \
        _get_vlan_network_interface_collection(connection["networkUri"])

    return ResponseBuilder.success(vlan_collection)


@vlan_network_interface.route(
    ResourceBlock.BASE_URI +
    "/<spt_uuid>/EthernetInterfaces/<connection_id>/VLANs", methods=["GET"])
def get_vlan_network_interface_collection_spt(spt_uuid, connection_id):
    """Get Redfish VLanNetworkInterfaceCollection for the given UUID and ID.

        Return VLanNetworkInterfaceCollection redfish JSON for a given
        ServerProfileTemplate UUID and connection ID.
        Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with VLanNetworkInterfaceCollection.
    """

    server_profile_template = \
        g.oneview_client.server_profile_templates.get_by_id(spt_uuid).data

    connection = \
        _get_connection_oneview_resource(server_profile_template,
                                         connection_id)

    vlan_collection = \
        _get_vlan_network_interface_collection(connection["networkUri"])

    return ResponseBuilder.success(vlan_collection)


def _get_connection_oneview_resource(oneview_resource, connection_id):
    """Get OneView Connection for the given resource and ID.

        Return OneView Connection a given OneView resource and connection ID.
        Logs exception of any error and Not Found.
        Returns:
            JSON: OneView Connection
    """

    conn_settings = oneview_resource["connectionSettings"]
    connection = None

    for conn in conn_settings["connections"]:
        if str(conn["id"]) == connection_id:
            connection = conn
            break

    if not connection:
        msg = "Ethernet interface not found"
        logging.exception('Unexpected error: {}'.format(msg))
        abort(status.HTTP_404_NOT_FOUND, msg)

    return connection


def _get_vlan_network_interface_collection(network_uri):
    """Get Redfish VLanNetworkInterfaceCollection for the given Network URI.

        Return VLanNetworkInterfaceCollection redfish JSON for a given
        Network Set URI.

        Returns:
            JSON: Redfish json with VLanNetworkInterfaceCollection.
    """

    network_set = g.oneview_client.network_sets.get_by_uri(network_uri).data

    vlan_collection = \
        VLanNetworkInterfaceCollection(network_set, request.path)

    return vlan_collection


@vlan_network_interface.route(
    ComputerSystem.BASE_URI +
    "/<sp_uuid>/EthernetInterfaces/<conn_id>/VLANs/<vlan_id>",
    methods=["GET"])
def get_vlan_network_interface_sp(sp_uuid, conn_id, vlan_id):
    """Get the Redfish VLanNetworkInterface for the given UUID, ID and VLanID.

        Return VLanNetowrkInterface redfish JSON for a given
        ServerProfileTemplate UUID, EthernetInterface ID and a VLan ID.
        Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with ResourceBlock.
    """

    server_profile = \
        g.oneview_client.server_profiles.get_by_id(sp_uuid).data

    _get_connection_oneview_resource(server_profile, conn_id)

    ethernet_network = \
        g.oneview_client.ethernet_networks.get_by_id(vlan_id).data

    vlan = VLanNetworkInterface(ethernet_network, request.path)

    return ResponseBuilder.success(vlan)


@vlan_network_interface.route(
    ResourceBlock.BASE_URI +
    "/<spt_uuid>/EthernetInterfaces/<conn_id>/VLANs/<vlan_id>",
    methods=["GET"])
def get_vlan_network_interface_spt(spt_uuid, conn_id, vlan_id):
    """Get the Redfish VLanNetworkInterface for the given UUID, ID and VLanID.

        Return VLanNetowrkInterface redfish JSON for a given
        ServerProfileTemplate UUID, EthernetInterface ID and a VLan ID.
        Logs exception of any error and return Internal Server
        Error or Not Found.

        Returns:
            JSON: Redfish json with ResourceBlock.
    """

    server_profile_template = \
        g.oneview_client.server_profile_templates.get_by_id(spt_uuid).data

    _get_connection_oneview_resource(server_profile_template, conn_id)

    ethernet_network = \
        g.oneview_client.ethernet_networks.get_by_id(vlan_id).data

    vlan = VLanNetworkInterface(ethernet_network, request.path)

    return ResponseBuilder.success(vlan)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from flask import Blueprint
from flask import g

from oneview_redfish_toolkit.api.zone_collection import ZoneCollection
from oneview_redfish_toolkit.blueprints.util.response_builder import \
    ResponseBuilder
from oneview_redfish_toolkit.config import \
    COUNTER_LOGGER_NAME
from oneview_redfish_toolkit.services import logging_service
from oneview_redfish_toolkit.services.zone_service import ZoneService

zone_collection = Blueprint("zone_collection", __name__)


@zone_collection.route(ZoneCollection.BASE_URI + "/", methods=["GET"])
def get_zone_collection():
    """Get the Redfish Zone Collection.

        Return ZoneCollection redfish JSON.

        Logs exception of any error and return Internal
        Server Error or Not Found.

        Returns:
            JSON: Redfish json with ZoneCollection.
    """

    server_profile_templates = \
        g.oneview_client.server_profile_templates.get_all()

    zone_service = ZoneService(g.oneview_client)
    zone_ids = zone_service.get_zone_ids_by_templates(server_profile_templates)

    zc = ZoneCollection(zone_ids)

    spt_count = len(server_profile_templates)
    zone_count = len(zc.redfish["Members"])
    logging_service.debug(
        COUNTER_LOGGER_NAME,
        "ServerProfileTemplates retrieved: " + str(spt_count),
        "Zones listed: " + str(zone_count))

    return ResponseBuilder.success(zc)



#!./redfish-venv/bin/python
# -*- coding: utf-8 -*-

# Copyright (2017) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections

from oneview_redfish_toolkit.api.redfish_json_validator import \
    RedfishJsonValidator


class Odata(RedfishJsonValidator):
    """Creates a hardcoded odata redfish dict

        Populates self.redfish with a hardcoded odata values. These should
        have all service offered by the servers.

    """

    SCHEMA_NAME = None

    def __init__(self):
        """Constructor

            Populates self.redfish with a hardcoded odata values
        """

        super().__init__(self.SCHEMA_NAME)
        self.redfish["@odata.context"] = "/redfish/v1/$metadata"
        self.redfish["value"] = list()
        self._append_item_to_value_list(
            name="Service",
            kind="Singleton",
            url="/redfish/v1/"
        )
        self._append_item_to_value_list(
            name="Systems",
            kind="Singleton",
            url="/redfish/v1/Systems"
        )
        self._append_item_to_value_list(
            name="Chassis",
            kind="Singleton",
            url="/redfish/v1/Chassis"
        )
        self._append_item_to_value_list(
            name="Managers",
            kind="Singleton",
            url="/redfish/v1/Managers"
        )

    def _append_item_to_value_list(self, name, kind, url):
        od = collections.OrderedDict()
        od["name"] = name
        od["kind"] = kind
        od["url"] = url
        self.redfish["value"].append(od)



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oneview_redfish_toolkit.api.redfish_json_validator \
    import RedfishJsonValidator


class ResourceBlock(RedfishJsonValidator):
    """Creates a ResourceBlock Redfish dict

        Populates self.redfish with some hardcoded ResourceBlock
        values and data retrieved from OneView.
    """

    SCHEMA_NAME = 'ResourceBlock'
    BASE_URI = '/redfish/v1/CompositionService/ResourceBlocks'

    def __init__(self, uuid, oneview_resource):
        """ResourceBlock constructor

            Populates self.redfish with the some common contents
            and data from OneView resource.

            Args:
                uuid: OneView resource UUID
                oneview_resource: resource dict from OneView
        """
        super().__init__(self.SCHEMA_NAME)

        self.uuid = uuid

        self.redfish["@odata.type"] = self.get_odata_type()
        self.redfish["Id"] = uuid
        self.redfish["Name"] = oneview_resource["name"]

        # This object must be completed by each implementation
        self.redfish["CompositionStatus"] = dict()

        self.redfish["@odata.context"] = \
            "/redfish/v1/$metadata#ResourceBlock.ResourceBlock"
        self.redfish["@odata.id"] = self.BASE_URI + "/" + uuid



# -*- coding: utf-8 -*-

# Copyright (2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

# 3rd party libs

from flask import Blueprint
from flask import g

# Own libs
from oneview_redfish_toolkit.api.processor_collection \
    import ProcessorCollection
from oneview_redfish_toolkit.api.resource_block_collection \
    import ResourceBlockCollection
from oneview_redfish_toolkit.blueprints.util.response_builder \
    import ResponseBuilder

processor_collection = Blueprint("processor_collection", __name__)


@processor_collection.route(
    ResourceBlockCollection.BASE_URI + "/<uuid>/Systems/1/Processors/",
    methods=["GET"])
def get_processor_collection(uuid):
    """Get the Redfish Resource Block System Processor Collection.

        Get method to return Resource Block System Processor Collection JSON.

        Returns:
            JSON: JSON with Resource Block System Processor Collection info.
    """

    server_hardware = g.oneview_client.server_hardware.get_by_id(uuid).data

    processor_collection = ProcessorCollection(server_hardware)

    return ResponseBuilder.success(
        processor_collection,
        {"ETag": "W/" + server_hardware["eTag"]})



# -*- coding: utf-8 -*-

# Copyright (2017-2018) Hewlett Packard Enterprise Development LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import collections
from oneview_redfish_toolkit.api.chassis \
    import Chassis

import re


class EnclosureChassis(Chassis):
    """Creates an Enclosure Chassis Redfish dict

         Populates self.redfish with some hardcoded Enclosure Chassis
         values and with the response of OneView enclosure resources.
    """

    def __init__(self, enclosure, environmental_configuration, manager_uuid):
        """Enclosure Chassis constructor

        Populates self.redfish with hardcoded Enclosure Chassis
        values and with the response of OneView enclosure.

        Args:
            enclosure: An object containing the Oneview enclosure
            to create the Redfish JSON.

            environmental_configuration: An object having information
            about the rack that containing the enclosure.
            manager_uuid: Oneview's current manager uuid.
        """

        super().__init__(enclosure)

        self.redfish["ChassisType"] = "Enclosure"
        self.redfish["Model"] = enclosure["enclosureModel"]
        self.redfish["IndicatorLED"] = self. \
            _map_indicator_led(enclosure["uidState"])
        self.redfish["Links"]["Contains"] = list()
        self._set_links_to_computer_system(
            enclosure["deviceBays"])
        self.redfish["Links"]["ManagedBy"] = list()
        if manager_uuid:
            self.redfish["Links"]["ManagedBy"].append(
                collections.OrderedDict())
            self.redfish["Links"]["ManagedBy"][0]["@odata.id"] = \
                "/redfish/v1/Managers/" + manager_uuid

        if environmental_configuration["rackId"]:
            self.redfish["Links"]["ContainedBy"] = {
                "@odata.id": "/redfish/v1/Chassis/" +
                             environmental_configuration["rackId"]
            }

        self._validate()

    def _set_links_to_computer_system(self, oneview_device_bays):
        """Mounts the list of Enclosure Links

            Populates self.redfish["Links"]["Contains"] with the links
            to all ComputerSystem chassis it contains.

            Args:
                oneview_device_bays: list of dicts containing information
                about devices and URI to OneView server hardware.
        """

        computer_systems_uuid = self. \
            _filter_by_computer_system_uuid(oneview_device_bays)

        for computer_system_uuid in computer_systems_uuid:
            link_dict = collections.OrderedDict()
            link_dict["@odata.id"] = \
                "/redfish/v1/Chassis/" + computer_system_uuid
            self.redfish["Links"]["Contains"].append(link_dict)

    def _filter_by_computer_system_uuid(self, oneview_device_bays):
        """Return Computer Systems UUID

            Iterate over oneview_device_bays and filter by only UUIDs
            related to server hardware.

            Args:
                oneview_device_bays: list of dicts containing information
                about devices and URI of OneView server hardware.

            Returns:
                list: List of ComputerSystem UUID.
        """

        pattern = re.compile(r'\b[A-Z0-9]{8}-[A-Z0-9]{4}-[A-Z0-9]{4}-'
                             r'[A-Z0-9]{4}-[A-Z0-9]{12}\b')

        computer_system_uuid = list()

        for device_dict in oneview_device_bays:
            uri = device_dict["deviceUri"]

            if uri is not None:
                uuid = uri.split("/")[-1]

                if pattern.match(uuid):
                    computer_system_uuid.append(uuid)

        return computer_system_uuid
